#  Copyright (c) 2015, Novartis Institutes for BioMedical Research Inc.
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import copy
import itertools
import os
import pickle
import sys
import time
import unittest

import numpy as np

from rdkit import Chem, Geometry, RDConfig, rdBase
from rdkit.Chem import AllChem, rdChemReactions


def log(s):
  rdBase.LogErrorMsg("== " + s)


class TestCase(unittest.TestCase):

  def setUp(self):
    self.dataDir = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'ChemReactions', 'testData')

  def testCartesianProduct(self):
    log("testCartesianProduct")
    rxn = rdChemReactions.ChemicalReaction()
    rgroups = [[Chem.MolFromSmiles("C")] * 10, [Chem.MolFromSmiles("N")] * 5,
               [Chem.MolFromSmiles("O")] * 6]

    cartProd = rdChemReactions.CartesianProductStrategy()
    cartProd.Initialize(rxn, rgroups)
    self.assertEqual(cartProd.GetNumPermutations(), 10 * 5 * 6)
    groups = []
    count = 0
    print(cartProd.__bool__())
    while cartProd:
      groups.append(tuple(cartProd.next()))


#      count += 1
#      assert count <= cartProd.GetNumPermutations()
    self.assertEqual(len(groups), 10 * 5 * 6)
    # see if we are equal to the Python implementation
    g = list(itertools.product(list(range(10)), list(range(5)), list(range(6))))
    self.assertEqual(set(g), set(groups))
    copy.copy(cartProd)

  def testRandomSample(self):
    log("testRandomSample")
    rgroups = [[Chem.MolFromSmiles("C")] * 10, [Chem.MolFromSmiles("N")] * 5,
               [Chem.MolFromSmiles("O")] * 6]
    rxn = rdChemReactions.ChemicalReaction()

    randProd = rdChemReactions.RandomSampleStrategy()
    randProd.Initialize(rxn, rgroups)
    self.assertEqual(randProd.GetNumPermutations(), 10 * 5 * 6)
    groups = []
    for i in range(10 * 5 * 6):
      groups.append(tuple(randProd.next()))
    print(len(set(groups)), "out of", 10 * 5 * 6)

    randProd = rdChemReactions.RandomSampleStrategy()
    randProd.Initialize(rxn, rgroups)
    self.assertEqual(randProd.GetNumPermutations(), 10 * 5 * 6)
    groups = []
    for i in range(10):
      groups.append(tuple(randProd.next()))

    for i in range(3):
      print(i, len(set([g[i] for g in groups])), "out of", [10, 5, 6][i])
    copy.copy(randProd)

  def testRandomSampleAllBBs(self):
    log("testRandomSampleAllBBs")
    rxn = rdChemReactions.ChemicalReaction()
    rgroups = [[Chem.MolFromSmiles("C")] * 10, [Chem.MolFromSmiles("N")] * 5,
               [Chem.MolFromSmiles("O")] * 6]

    randProd = rdChemReactions.RandomSampleAllBBsStrategy()
    randProd.Initialize(rxn, rgroups)
    self.assertEqual(randProd.GetNumPermutations(), 10 * 5 * 6)
    groups = []
    for i in range(10 * 5 * 6):
      groups.append(tuple(randProd.next()))

    print(len(set(groups)), "out of", 10 * 5 * 6)

    randProd = rdChemReactions.RandomSampleAllBBsStrategy()
    randProd.Initialize(rxn, rgroups)
    self.assertEqual(randProd.GetNumPermutations(), 10 * 5 * 6)
    groups = []
    for i in range(10):
      groups.append(tuple(randProd.next()))

    for i in range(3):
      print(i, len(set([g[i] for g in groups])), "out of", [10, 5, 6][i])
      self.assertEqual(len(set([g[i] for g in groups])), [10, 5, 6][i])
    copy.copy(randProd)

  def testTimings(self):
    log("testTimings")
    rxn = rdChemReactions.ChemicalReaction()

    rgroups = [[Chem.MolFromSmiles("C")] * 17000, [Chem.MolFromSmiles("N")] * 50000,
               [Chem.MolFromSmiles("O")] * 4000]
    cartProd = rdChemReactions.CartesianProductStrategy()
    randProd = rdChemReactions.RandomSampleStrategy()
    randAllBBs = rdChemReactions.RandomSampleAllBBsStrategy()
    for r in [cartProd, randProd, randAllBBs]:
      r.Initialize(rxn, rgroups)
      num = 10000000
      t1 = time.time()
      r.Skip(num)
      t2 = time.time()
      print("%s Skipped %s in %s seconds" % (r, num, t2 - t1))

  def testEvenPairsSampling(self):
    rxn = rdChemReactions.ChemicalReaction()

    rgroups = [[Chem.MolFromSmiles("C")] * 10, [Chem.MolFromSmiles("N")] * 10,
               [Chem.MolFromSmiles("O")] * 10]

    rxn = rdChemReactions.ChemicalReaction()
    count = 0
    pairs01 = {}
    pairs12 = {}
    pairs02 = {}

    strategy = rdChemReactions.EvenSamplePairsStrategy()
    strategy.Initialize(rxn, rgroups)
    # try 100 samples
    while count < 100:
      v = strategy.next()
      p01 = (v[0], v[1])
      p12 = (v[1], v[2])
      p02 = (v[0], v[2])
      pairs01[p01] = pairs01.get(p01, 0) + 1
      pairs12[p01] = pairs12.get(p12, 0) + 1
      pairs02[p01] = pairs02.get(p02, 0) + 1
      count += 1

    # each pair should be used roughly once
    self.assertEqual(np.median(list(pairs01.values())), 1.0)
    self.assertEqual(np.median(list(pairs02.values())), 1.0)
    self.assertEqual(np.median(list(pairs12.values())), 1.0)

    # now try 1000
    pairs01 = {}
    pairs12 = {}
    pairs02 = {}
    strategy = rdChemReactions.EvenSamplePairsStrategy()
    strategy.Initialize(rxn, rgroups)
    count = 0
    while count < 1000:
      v = strategy.next()
      p01 = (v[0], v[1])
      p12 = (v[1], v[2])
      p02 = (v[0], v[2])
      pairs01[p01] = pairs01.get(p01, 0) + 1
      pairs12[p01] = pairs12.get(p12, 0) + 1
      pairs02[p01] = pairs02.get(p02, 0) + 1
      count += 1

    # each pair should be used roughly 10 times
    self.assertTrue(9 <= np.median(list(pairs01.values())) <= 11)
    self.assertTrue(9 <= np.median(list(pairs02.values())) <= 11)
    self.assertTrue(9 <= np.median(list(pairs12.values())) <= 11)

    # now try 500
    pairs01 = {}
    pairs12 = {}
    pairs02 = {}
    strategy = rdChemReactions.EvenSamplePairsStrategy()
    strategy.Initialize(rxn, rgroups)
    count = 0
    while count < 500:
      v = strategy.next()
      p01 = (v[0], v[1])
      p12 = (v[1], v[2])
      p02 = (v[0], v[2])
      pairs01[p01] = pairs01.get(p01, 0) + 1
      pairs12[p01] = pairs12.get(p12, 0) + 1
      pairs02[p01] = pairs02.get(p02, 0) + 1
      count += 1

    # each pair should be used roughly 5 times
    self.assertTrue(4 <= np.median(list(pairs01.values())) <= 6)
    self.assertTrue(4 <= np.median(list(pairs02.values())) <= 6)
    self.assertTrue(4 <= np.median(list(pairs12.values())) <= 6)

    self.assertTrue("PAIRSTAT" in strategy.Stats())

  def testEnumerateLibrary(self):
    log("testEnumerateLibrary")
    smirks_thiourea = "[N;$(N-[#6]):3]=[C;$(C=S):1].[N;$(N[#6]);!$(N=*);!$([N-]);!$(N#*);!$([ND3]);!$([ND4]);!$(N[O,N]);!$(N[C,S]=[S,O,N]):2]>>[N:3]-[C:1]-[N+0:2]"
    rxn = rdChemReactions.ReactionFromSmarts(smirks_thiourea)
    reagents = [[Chem.MolFromSmiles('C=CCN=C=S'),
                 Chem.MolFromSmiles('CC=CCN=C=S')],
                [
                  Chem.MolFromSmiles('NCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCCc1ncc(Cl)cc1Br'),
                ]]

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents)
    self.assertTrue(enumerator)

    # need to initialize the reaction before getting the binary serialization
    rxn.Initialize()
    self.assertEqual(rxn.ToBinary(), enumerator.GetReaction().ToBinary())

    bbs = enumerator.GetReagents()
    for i in range(len(bbs)):
      for j in range(len(bbs[i])):
        self.assertTrue(Chem.MolToSmiles(reagents[i][j]) == Chem.MolToSmiles(bbs[i][j]))

    smiresults = [
      'C=CCNC(=S)NCc1ncc(Cl)cc1Br', 'CC=CCNC(=S)NCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCCc1ncc(Cl)cc1Br'
    ]
    results = [Chem.MolToSmiles(Chem.MolFromSmiles(smi)) for smi in smiresults]

    enumerators = [enumerator]

    # add serialized enumerators as well for testing if possible
    if rdChemReactions.EnumerateLibraryCanSerialize():
      pickle = enumerator.Serialize()
      enumerator2 = rdChemReactions.EnumerateLibrary()
      enumerator2.InitFromString(pickle)

      # make sure old pickles work
      enumerator3 = rdChemReactions.EnumerateLibrary()
      enumerator3.InitFromString(
        open(os.path.join(self.dataDir, "enumeration.pickle"), 'rb').read())

      print("==", enumerator.GetEnumerator().Type(), enumerator2.GetEnumerator().Type())
      self.assertEqual(enumerator.GetEnumerator().Type(), enumerator2.GetEnumerator().Type())
      enumerators.append(enumerator2)
      enumerators.append(enumerator3)

    # check for fully sampled and deterministic ordering in final index values
    expected_positions = [[0, 0], [1, 0], [0, 1], [1, 1], [0, 2], [1, 2]]

    out = []
    for en in enumerators:
      i = 0
      positions = []
      for i, prods in enumerate(en):
        positions.append(list(en.GetPosition()))
        for mols in prods:
          self.assertEqual(len(mols), 1)
          smi = Chem.MolToSmiles(mols[0])
          if en is enumerator:
            out.append(smi)
          self.assertEqual(smi, results[i])

        if en is enumerator and i == 1 and rdChemReactions.EnumerateLibraryCanSerialize():
          # save the state not at the start
          pickle_at_2 = enumerator.Serialize()
      self.assertEqual(i, 5)
      self.assertEqual(positions, expected_positions)

    if rdChemReactions.EnumerateLibraryCanSerialize():
      # see if we can restore the enumeration from the middle
      out3 = []
      enumerator3 = rdChemReactions.EnumerateLibrary()
      enumerator3.InitFromString(pickle_at_2)
      for prods in enumerator3:
        for mols in prods:
          self.assertEqual(len(mols), 1)
          smi = Chem.MolToSmiles(mols[0])
          out3.append(smi)

      self.assertEqual(out[2:], out3)
    # test smiles interface
    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents)
    i = 0
    while enumerator:
      for mols in enumerator.nextSmiles():
        self.assertEqual(len(mols), 1)
        self.assertEqual(mols[0], results[i])
      i += 1
    self.assertEqual(i, 6)

  def testRandomEnumerateLibrary(self):
    log("testRandomEnumerateLibrary")
    smirks_thiourea = "[N;$(N-[#6]):3]=[C;$(C=S):1].[N;$(N[#6]);!$(N=*);!$([N-]);!$(N#*);!$([ND3]);!$([ND4]);!$(N[O,N]);!$(N[C,S]=[S,O,N]):2]>>[N:3]-[C:1]-[N+0:2]"
    rxn = rdChemReactions.ReactionFromSmarts(smirks_thiourea)
    reagents = [[Chem.MolFromSmiles('C=CCN=C=S'),
                 Chem.MolFromSmiles('CC=CCN=C=S')],
                [
                  Chem.MolFromSmiles('NCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCCc1ncc(Cl)cc1Br'),
                ]]

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                  rdChemReactions.RandomSampleStrategy())
    self.assertTrue(enumerator)
    smiresults = [
      'C=CCNC(=S)NCc1ncc(Cl)cc1Br', 'CC=CCNC(=S)NCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCCc1ncc(Cl)cc1Br'
    ]
    results = [Chem.MolToSmiles(Chem.MolFromSmiles(smi)) for smi in smiresults]

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                  rdChemReactions.RandomSampleStrategy())
    iteren = iter(enumerator)
    res = set()
    count = 0
    while res != set(results):
      count += 1
      if count > 100000:
        print("Unable to find enumerate set with 100,000 random samples!", file=sys.stderr)
        self.assertEqual(res, set(results))

      prod = iteren.next()
      for mols in prod:
        smi1 = Chem.MolToSmiles(mols[0])
        res.add(smi1)

    if rdChemReactions.EnumerateLibraryCanSerialize():
      enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                    rdChemReactions.RandomSampleStrategy())
      pickle = enumerator.Serialize()
      enumerator2 = rdChemReactions.EnumerateLibrary()
      enumerator2.InitFromString(pickle)

      self.assertEqual(enumerator.GetEnumerator().Type(), enumerator2.GetEnumerator().Type())

      iteren = iter(enumerator)
      iteren2 = iter(enumerator2)

      outsmiles = []
      for i in range(10):
        prods1 = iteren.next()
        prods2 = iteren2.next()
        self.assertEqual(len(prods1), len(prods2))
        for mols1, mols2 in zip(prods1, prods2):
          self.assertEqual(len(mols1), 1)
          smi1 = Chem.MolToSmiles(mols1[0])
          self.assertEqual(smi1, Chem.MolToSmiles(mols2[0]))
          outsmiles.append(smi1)

        if i == 1:
          pickle_at_2 = enumerator.Serialize()

      # make sure we can pickle the state as well
      enumerator3 = rdChemReactions.EnumerateLibrary()
      enumerator3.InitFromString(pickle_at_2)
      iteren3 = iter(enumerator3)
      outsmiles2 = []
      for i in range(8):
        prods3 = iteren3.next()
        for mols3 in prods3:
          self.assertEqual(len(mols3), 1)
          smi1 = Chem.MolToSmiles(mols3[0])
          self.assertEqual(smi1, Chem.MolToSmiles(mols3[0]))
          outsmiles2.append(smi1)

      self.assertEqual(outsmiles2, outsmiles[2:])

  def testRandomEnumerateAllBBsLibrary(self):
    log("testRandomEnumerateAllBBsLibrary")
    smirks_thiourea = "[N;$(N-[#6]):3]=[C;$(C=S):1].[N;$(N[#6]);!$(N=*);!$([N-]);!$(N#*);!$([ND3]);!$([ND4]);!$(N[O,N]);!$(N[C,S]=[S,O,N]):2]>>[N:3]-[C:1]-[N+0:2]"
    rxn = rdChemReactions.ReactionFromSmarts(smirks_thiourea)
    reagents = [[Chem.MolFromSmiles('C=CCN=C=S'),
                 Chem.MolFromSmiles('CC=CCN=C=S')],
                [
                  Chem.MolFromSmiles('NCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCCc1ncc(Cl)cc1Br'),
                ]]
    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                  rdChemReactions.RandomSampleAllBBsStrategy())
    self.assertTrue(enumerator)

    # test the BB sampling here
    strategy = iter(enumerator)
    r1 = set()
    r2 = set()
    strategy.next()
    groups = strategy.GetPosition()
    print("**", list(groups), file=sys.stderr)
    r1.add(groups[0])
    r2.add(groups[1])
    strategy.next()
    groups = strategy.GetPosition()
    print("**", list(groups), file=sys.stderr)
    r1.add(groups[0])
    r2.add(groups[1])
    self.assertEqual(r1, set([0, 1]))  # two bbs at reagent one all sampled at one iteration
    strategy.next()
    groups = strategy.GetPosition()
    print("**", list(groups), file=sys.stderr)
    r1.add(groups[0])
    r2.add(groups[1])
    self.assertEqual(r2, set([0, 1,
                              2]))  # three bbs at reagent one all sampled in three iterations

    smiresults = [
      'C=CCNC(=S)NCc1ncc(Cl)cc1Br', 'CC=CCNC(=S)NCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCCc1ncc(Cl)cc1Br'
    ]
    results = [Chem.MolToSmiles(Chem.MolFromSmiles(smi)) for smi in smiresults]

    if rdChemReactions.EnumerateLibraryCanSerialize():
      enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                    rdChemReactions.RandomSampleAllBBsStrategy())
      self.assertTrue(enumerator)

      pickle = enumerator.Serialize()
      enumerator2 = rdChemReactions.EnumerateLibrary()
      enumerator2.InitFromString(pickle)

      self.assertEqual(enumerator.GetEnumerator().Type(), enumerator2.GetEnumerator().Type())
      iteren = iter(enumerator)
      iteren2 = iter(enumerator2)

      outsmiles = []
      for i in range(10):
        prods1 = iteren.next()
        prods2 = iteren2.next()
        self.assertEqual(len(prods1), len(prods2))
        for mols1, mols2 in zip(prods1, prods2):
          self.assertEqual(len(mols1), 1)
          smi1 = Chem.MolToSmiles(mols1[0])
          self.assertEqual(smi1, Chem.MolToSmiles(mols2[0]))
          outsmiles.append(smi1)

        if i == 1:
          pickle_at_2 = enumerator.Serialize()

      # make sure we can pickle the state as well
      enumerator3 = rdChemReactions.EnumerateLibrary()
      enumerator3.InitFromString(pickle_at_2)
      self.assertEqual(enumerator.GetEnumerator().Type(), enumerator3.GetEnumerator().Type())

      iteren3 = iter(enumerator3)
      outsmiles2 = []
      for i in range(8):
        prods3 = iteren3.next()
        for mols3 in prods3:
          self.assertEqual(len(mols3), 1)
          smi1 = Chem.MolToSmiles(mols3[0])
          self.assertEqual(smi1, Chem.MolToSmiles(mols3[0]))
          outsmiles2.append(smi1)

      self.assertEqual(outsmiles2, outsmiles[2:])

  def testRGroupState(self):
    if not rdChemReactions.EnumerateLibraryCanSerialize():
      print("-- Skipping testRGroupState, serialization of EnumerateLibrary not enabled",
            file=sys.stderr)
      return

    log("testRGroupState")
    smirks_thiourea = "[N;$(N-[#6]):3]=[C;$(C=S):1].[N;$(N[#6]);!$(N=*);!$([N-]);!$(N#*);!$([ND3]);!$([ND4]);!$(N[O,N]);!$(N[C,S]=[S,O,N]):2]>>[N:3]-[C:1]-[N+0:2]"
    rxn = rdChemReactions.ReactionFromSmarts(smirks_thiourea)
    reagents = [[Chem.MolFromSmiles('C=CCN=C=S'),
                 Chem.MolFromSmiles('CC=CCN=C=S')],
                [
                  Chem.MolFromSmiles('NCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCc1ncc(Cl)cc1Br'),
                  Chem.MolFromSmiles('NCCCc1ncc(Cl)cc1Br'),
                ]]

    def tostr(l):
      return [[str(x) for x in v] for v in l]

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents)
    state = enumerator.GetState()
    p = enumerator.nextSmiles()
    p2 = enumerator.nextSmiles()
    enumerator.SetState(state)
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p))
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p2))

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                  rdChemReactions.RandomSampleStrategy())

    state = enumerator.GetState()
    p = enumerator.nextSmiles()
    p2 = enumerator.nextSmiles()
    enumerator.SetState(state)
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p))
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p2))

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents,
                                                  rdChemReactions.RandomSampleAllBBsStrategy())
    state = enumerator.GetState()
    p = enumerator.nextSmiles()
    p2 = enumerator.nextSmiles()
    enumerator.SetState(state)
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p))
    self.assertEqual(tostr(enumerator.nextSmiles()), tostr(p2))

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents)
    smiresults = [
      'C=CCNC(=S)NCc1ncc(Cl)cc1Br', 'CC=CCNC(=S)NCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCc1ncc(Cl)cc1Br', 'C=CCNC(=S)NCCCc1ncc(Cl)cc1Br',
      'CC=CCNC(=S)NCCCc1ncc(Cl)cc1Br'
    ]
    smiresults = [Chem.MolToSmiles(Chem.MolFromSmiles(smi)) for smi in smiresults]
    enumerator.GetEnumerator().Skip(10)
    enumerator.ResetState()

    results = []
    for result in enumerator:
      for prodSet in result:
        for mol in prodSet:
          results.append(Chem.MolToSmiles(mol))

    self.assertEqual(results, smiresults)

  def testRemovingBadMatches(self):
    log("testRemoveBadMatches")
    smirks_thiourea = "[N;$(N-[#6]):3]=[C;$(C=S):1].[N;$(N[#6]);!$(N=*);!$([N-]);!$(N#*);!$([ND3]);!$([ND4]);!$(N[O,N]);!$(N[C,S]=[S,O,N]):2]>>[N:3]-[C:1]-[N+0:2]"

    rxn = rdChemReactions.ReactionFromSmarts(smirks_thiourea)
    # invert matches so nothing matches
    reagents = [
      [
        Chem.MolFromSmiles('NCc1ncc(Cl)cc1Br'),
        Chem.MolFromSmiles('NCCc1ncc(Cl)cc1Br'),
        Chem.MolFromSmiles('NCCCc1ncc(Cl)cc1Br'),
      ],
      [
        Chem.MolFromSmiles('C=CCN=C=S'),
        Chem.MolFromSmiles('CC=CCN=C=S'),
        Chem.MolFromSmiles('CCC'),
        Chem.MolFromSmiles('CCCCC'),
      ],
    ]

    enumerator = rdChemReactions.EnumerateLibrary(rxn, reagents)
    self.assertEqual([], list(enumerator))

  def testRemoveInsaneReagents(self):
    rxndata = "$RXN\nUntitled Document-1\n  ChemDraw10291618492D\n\n  3  1\n$MOL\n\n\n\n  2  1  0  0  0  0  0  0  0  0999 V2000\n    0.4125    0.0000    0.0000 N   0  0  0  0  0  0  0  0  0  3  0  0\n   -0.4125    0.0000    0.0000 R2  0  0  0  0  0  0  0  0  0  2  0  0\n  1  2  1  0        0\nM  END\n$MOL\n\n\n\n  2  1  0  0  0  0  0  0  0  0999 V2000\n   -0.4125    0.0000    0.0000 R1  0  0  0  0  0  0  0  0  0  1  0  0\n    0.4125    0.0000    0.0000 Cl  0  0  0  0  0  0  0  0  0  0  0  0\n  1  2  1  0        0\nM  END\n$MOL\n\n\n\n  2  1  0  0  0  0  0  0  0  0999 V2000\n    0.4125    0.0000    0.0000 N   0  0  0  0  0  0  0  0  0  5  0  0\n   -0.4125    0.0000    0.0000 R4  0  0  0  0  0  0  0  0  0  4  0  0\n  1  2  1  0        0\nM  END\n$MOL\n\n\n\n 14 15  0  0  0  0  0  0  0  0999 V2000\n    0.5072   -0.5166    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n    0.5072    0.3084    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n    1.2949   -0.7616    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0\n    1.7817   -0.0880    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n    1.2967    0.5794    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n    1.5558   -1.5443    0.0000 R1  0  0  0  0  0  0  0  0  0  1  0  0\n   -0.2073    0.7208    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -0.9218    0.3083    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -0.9217   -0.5167    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -0.2073   -0.9292    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -1.6362    0.7208    0.0000 N   0  0  0  0  0  0  0  0  0  3  0  0\n    1.5452    1.3661    0.0000 N   0  0  0  0  0  0  0  0  0  5  0  0\n    2.3507    1.5443    0.0000 R4  0  0  0  0  0  0  0  0  0  4  0  0\n   -2.3507    0.3083    0.0000 R2  0  0  0  0  0  0  0  0  0  2  0  0\n  1  2  2  0        0\n  1  3  1  0        0\n  3  4  1  0        0\n  4  5  1  0        0\n  5  2  1  0        0\n  3  6  1  0        0\n  2  7  1  0        0\n  7  8  2  0        0\n  8  9  1  0        0\n  9 10  2  0        0\n 10  1  1  0        0\n  8 11  1  0        0\n 12 13  1  0        0\n 11 14  1  0        0\n 12  5  1  0        0\nM  END\n"

    rxn = AllChem.ReactionFromRxnBlock(rxndata)
    bbs = []
    r1 = [
      Chem.MolFromSmiles("CCNCC"),
      Chem.MolFromSmiles("NCC"),
    ]
    r2 = [
      Chem.MolFromSmiles("ClC1CCCC1"),
      Chem.MolFromSmiles("ClC1CCCC1Cl"),
    ]
    r3 = [
      Chem.MolFromSmiles("CCNCC"),
      Chem.MolFromSmiles("NCC"),
    ]
    bbs = [r1, r2, r3]

    # nothing matches!
    for i, reagent in enumerate(rxn.GetReactants()):
      for bb in bbs[i]:
        self.assertFalse(bb.HasSubstructMatch(reagent))

    # everything matches - yay sanitization!
    rdChemReactions.SanitizeRxn(rxn)
    for i, reagent in enumerate(rxn.GetReactants()):
      for bb in bbs[i]:
        self.assertTrue(bb.HasSubstructMatch(reagent))

    en = rdChemReactions.EnumerateLibrary(rxn, bbs)
    self.assertTrue(len(en.GetReagents()[0]) == 2)
    self.assertTrue(len(en.GetReagents()[1]) == 2)
    self.assertTrue(len(en.GetReagents()[2]) == 2)

    #####################################################################################
    # Match only at rgroups (ChemDraw style)
    rxn = AllChem.ReactionFromRxnBlock(rxndata)
    expected_matches = [[False, True], [True, True], [False, True]]
    rdChemReactions.SanitizeRxn(rxn, params=rdChemReactions.GetChemDrawRxnAdjustParams())
    for i, (reagent, expected) in enumerate(zip(rxn.GetReactants(), expected_matches)):
      match = [bb.HasSubstructMatch(reagent) for reagent in bbs[i]]
      self.assertTrue(match, expected)

    # Now try EnumerateLibrary
    en = rdChemReactions.EnumerateLibrary(rxn, bbs)
    self.assertTrue(len(en.GetReagents()[0]) == 1)
    self.assertTrue(len(en.GetReagents()[1]) == 2)
    self.assertTrue(len(en.GetReagents()[2]) == 1)

    #####################################################################################
    # now set the removal options ot only make one product per reagent set
    rxn = AllChem.ReactionFromRxnBlock(rxndata)
    rdChemReactions.SanitizeRxn(rxn)

    opts = rdChemReactions.EnumerationParams()
    opts.reagentMaxMatchCount = 1
    en = rdChemReactions.EnumerateLibrary(rxn, bbs, params=opts)
    self.assertTrue(len(en.GetReagents()[0]) == 1)
    self.assertTrue(len(en.GetReagents()[1]) == 1)
    self.assertTrue(len(en.GetReagents()[2]) == 1)

    #####################################################################################
    # now set the removal options ot only make one product per reagent set
    #  but wt
    rxn = AllChem.ReactionFromRxnBlock(rxndata)
    rdChemReactions.SanitizeRxn(rxn, params=rdChemReactions.GetChemDrawRxnAdjustParams())

    opts = rdChemReactions.EnumerationParams()
    opts.reagentMaxMatchCount = 1
    en = rdChemReactions.EnumerateLibrary(rxn, bbs, params=opts)
    self.assertTrue(len(en.GetReagents()[0]) == 1)
    self.assertTrue(len(en.GetReagents()[1]) == 1)
    self.assertTrue(len(en.GetReagents()[2]) == 1)

if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2003-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""
unit testing code for the Smiles file handling stuff
"""
import unittest

from rdkit import Chem, RDLogger


class TestCase(unittest.TestCase):

  def setUp(self):
    self.smis = ['CC', 'CCC', 'CCCCC', 'CCCCCC', 'CCCCCCC', 'CC', 'CCCCOC']
    self.nMolecules = len(self.smis)

  def tearDown(self):
    RDLogger.EnableLog('rdApp.error')

  def assertMolecule(self, mol, i, msg=''):
    """ Assert that we have a valid molecule """
    self.assertIsNotNone(mol, '{0}read {1} failed'.format(msg, i))
    self.assertGreater(mol.GetNumAtoms(), 0, '{0}no atoms in mol {1}'.format(msg, i))

  def test_SmilesReaderIndex(self):
    # tests lazy reads
    supp = Chem.SmilesMolSupplierFromText('\n'.join(self.smis), ',', 0, -1, 0)
    for i in range(4):
      self.assertMolecule(next(supp), i)

    i = len(supp) - 1
    self.assertMolecule(supp[i], i)

    # Use in a list comprehension
    ms = [Chem.MolToSmiles(mol) for mol in supp]
    self.assertEqual(ms, self.smis)

    self.assertEqual(len(supp), self.nMolecules, 'bad supplier length')

    # Despite iterating through the whole supplier, we can still access by index
    i = self.nMolecules - 3
    self.assertMolecule(supp[i - 1], i, msg='back index: ')

    with self.assertRaises(IndexError):
      _ = supp[self.nMolecules]  # out of bound read must fail

    # and we can access with negative numbers
    mol1 = supp[len(supp) - 1]
    mol2 = supp[-1]
    self.assertEqual(Chem.MolToSmiles(mol1), Chem.MolToSmiles(mol2))

  def test_SmilesReaderIterator(self):
    # tests lazy reads using the iterator interface "
    supp = Chem.SmilesMolSupplierFromText('\n'.join(self.smis), ',', 0, -1, 0)

    nDone = 0
    for mol in supp:
      self.assertMolecule(mol, nDone)
      nDone += 1
    self.assertEqual(nDone, self.nMolecules, 'bad number of molecules')

    self.assertEqual(len(supp), self.nMolecules, 'bad supplier length')

    # Despite iterating through the whole supplier, we can still access by index
    i = self.nMolecules - 3
    self.assertMolecule(supp[i - 1], i, msg='back index: ')

    with self.assertRaises(IndexError):
      _ = supp[self.nMolecules]  # out of bound read must not fail

  def test_SmilesReaderBoundaryConditions(self):
    # Suppress the error message due to the incorrect smiles
    RDLogger.DisableLog('rdApp.error')

    smis = ['CC', 'CCOC', 'fail', 'CCO']
    supp = Chem.SmilesMolSupplierFromText('\n'.join(smis), ',', 0, -1, 0)
    self.assertEqual(len(supp), 4)
    self.assertIsNone(supp[2])
    self.assertIsNotNone(supp[3])

    supp = Chem.SmilesMolSupplierFromText('\n'.join(smis), ',', 0, -1, 0)
    self.assertIsNone(supp[2])
    self.assertIsNotNone(supp[3])
    self.assertEqual(len(supp), 4)
    with self.assertRaises(IndexError):
      supp[4]

    supp = Chem.SmilesMolSupplierFromText('\n'.join(smis), ',', 0, -1, 0)
    self.assertEqual(len(supp), 4)
    self.assertIsNotNone(supp[3])
    with self.assertRaises(IndexError):
      supp[4]

    supp = Chem.SmilesMolSupplierFromText('\n'.join(smis), ',', 0, -1, 0)
    with self.assertRaises(IndexError):
      supp[4]

    self.assertEqual(len(supp), 4)
    self.assertIsNotNone(supp[3])


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
#  Copyright (C) 2004-2006  Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" unit testing code for MOE-type descriptors with EStates

"""

import os
import unittest

from rdkit import Chem
from rdkit.Chem.EState import EState_VSA


class TestCase(unittest.TestCase):

  @staticmethod
  def referenceData():
    filename = os.sep.join(
      [os.path.dirname(os.path.abspath(__file__)), 'test_data', 'EState_VSA.csv'])
    with open(filename) as fin:
      header = fin.readline()
      header = [s.strip() for s in header.split(',')][1:]

      funcEstates = dict((k, getattr(EState_VSA, k)) for k in header)
      yield funcEstates

      for line in fin:
        line = [s.strip() for s in line.split(',')]
        smiles = line.pop(0)
        mol = Chem.MolFromSmiles(smiles)
        data = dict((k, float(v)) for k, v in zip(header, line))
        yield smiles, mol, data

  def test1(self):
    referenceData = self.referenceData()
    funcEstates = next(referenceData)
    for smiles, mol, data in referenceData:
      for name in funcEstates:
        calc = funcEstates[name](mol)
        exp = data[name]
        self.assertAlmostEqual(calc, exp, delta=1e-4,
                               msg='{0}: {1:.4f}!={2:.4f}'.format(smiles, calc, exp))


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import overloads

assert overloads.f1(1) == 4
assert overloads.f1(1, 4) == 5
assert overloads.f1(1.) == 4.
assert overloads.f1(1., 5) == 6.

assert overloads.f2(1) == 4
assert overloads.f2(1, 12) == 13



"""unit testing code for the DbResultSet object

"""

import random
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem.Suppliers.DbMolSupplier import (ForwardDbMolSupplier,
                                                RandomAccessDbMolSupplier)
from rdkit.Chem.Suppliers.MolSupplier import MolSupplier
from rdkit.Dbase.DbConnection import DbConnect
from rdkit.Dbase.DbResultSet import DbResultSet, RandomAccessDbResultSet


class TestCase(unittest.TestCase):

  def setUp(self):
    self.dbName = RDConfig.RDTestDatabase
    self.conn = DbConnect(self.dbName)
    self.curs = self.conn.GetCursor()

  def test_MolSupplier(self):
    self.assertRaises(ValueError, MolSupplier)

  def test_general(self):
    # Check for a molecule column
    cmd = 'select * from ten_elements'
    results = DbResultSet(self.curs, self.conn, cmd)
    self.assertRaises(ValueError, ForwardDbMolSupplier, results)

  def test_ForwardDbMolSupplier(self):
    cmd = 'select * from simple_mols order by ID'
    results = DbResultSet(self.curs, self.conn, cmd)
    expected = list(results)

    results = DbResultSet(self.curs, self.conn, cmd)
    supp = ForwardDbMolSupplier(results)
    self.assertEqual(supp.GetColumnNames(), ('ID', ))

    for smiles, mol in zip(expected, supp):
      self.assertEqual(Chem.MolToSmiles(Chem.MolFromSmiles(smiles[0])), Chem.MolToSmiles(mol))
      self.assertEqual(smiles[1], mol.GetProp('ID'))
    self.assertRaises(StopIteration, next, supp)

    # We can not use an index for ForwardDbMolSupplier
    with self.assertRaises(TypeError):
      supp[0]

  def test_RandomAccessDbMolSupplier(self):
    cmd = 'select * from simple_mols order by ID'
    results = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    expected = list(results)

    results = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    supp = RandomAccessDbMolSupplier(results)
    self.assertEqual(len(supp), len(expected))
    self.assertEqual(supp.GetColumnNames(), ('ID', ))
    for smiles, mol in zip(expected, supp):
      self.assertEqual(Chem.MolToSmiles(Chem.MolFromSmiles(smiles[0])), Chem.MolToSmiles(mol))
      self.assertEqual(smiles[1], mol.GetProp('ID'))

    # Check that we can randomly access the data
    indices = list(range(len(expected)))
    random.shuffle(indices)
    for idx in indices:
      smiles = expected[idx]
      mol = supp[idx]
      self.assertEqual(Chem.MolToSmiles(Chem.MolFromSmiles(smiles[0])), Chem.MolToSmiles(mol))
      self.assertEqual(smiles[1], mol.GetProp('ID'))

    # We get an error if we access outside of the permitted range
    with self.assertRaises(IndexError):
      supp[len(expected)]

    # The DbMolSupplier doesn't support negative indices
    with self.assertRaises(IndexError):
      supp[-1]


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (c) 2013, Novartis Institutes for BioMedical Research Inc.
#  Copyright (c) 2021, Greg Landrum
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Created by Sereina Riniker, Aug 2013
""" unit testing code for molecule drawing
"""

import sys
import unittest

from rdkit import Chem
from rdkit.RDLogger import logger

try:
  import matplotlib
except ImportError:
  matplotlib = None

from rdkit.Chem import Draw
from rdkit.Chem.Draw import rdMolDraw2D
from rdkit.Chem.Draw import SimilarityMaps as sm

logger = logger()


class TestCase(unittest.TestCase):

  def setUp(self):
    self.mol1 = Chem.MolFromSmiles('c1ccccc1')
    self.mol2 = Chem.MolFromSmiles('c1ccncc1')

  @unittest.skipUnless(matplotlib, 'Matplotlib required')
  def testSimilarityMap(self):
    # Morgan2 BV
    refWeights = [0.5, 0.5, 0.5, -0.5, 0.5, 0.5]
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'))
    for w, r in zip(weights, refWeights):
      self.assertEqual(w, r)

    d2d = rdMolDraw2D.MolDraw2DSVG(400, 400)
    _, maxWeight = sm.GetSimilarityMapForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'), d2d)
    self.assertEqual(maxWeight, 0.5)

    weights, maxWeight = sm.GetStandardizedWeights(weights)
    self.assertEqual(maxWeight, 0.5)
    refWeights = [1.0, 1.0, 1.0, -1.0, 1.0, 1.0]
    for w, r in zip(weights, refWeights):
      self.assertEqual(w, r)

    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetMorganFingerprint(m, i, fpType='count'))
    self.assertTrue(weights[3] < 0)
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2,
      lambda m, i: sm.GetMorganFingerprint(m, i, fpType='bv', useFeatures=True))
    self.assertTrue(weights[3] < 0)

    # hashed AP BV
    refWeights = [0.09523, 0.17366, 0.17366, -0.23809, 0.17366, 0.17366]
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetAPFingerprint(m, i, fpType='bv', nBits=1024))
    for w, r in zip(weights, refWeights):
      self.assertAlmostEqual(w, r, 4)

    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetAPFingerprint(m, i, fpType='normal'))
    self.assertTrue(weights[3] < 0)
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetAPFingerprint(m, i, fpType='hashed'))
    self.assertTrue(weights[3] < 0)

    # hashed TT BV
    refWeights = [0.5, 0.5, -0.16666, -0.5, -0.16666, 0.5]
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2,
      lambda m, i: sm.GetTTFingerprint(m, i, fpType='bv', nBits=1024, nBitsPerEntry=1))
    for w, r in zip(weights, refWeights):
      self.assertAlmostEqual(w, r, 4)

    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetTTFingerprint(m, i, fpType='normal'))
    self.assertTrue(weights[3] < 0)
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetTTFingerprint(m, i, fpType='hashed'))
    self.assertTrue(weights[3] < 0)

    # RDK fingerprint BV
    refWeights = [0.42105, 0.42105, 0.42105, -0.32895, 0.42105, 0.42105]
    weights = sm.GetAtomicWeightsForFingerprint(
      self.mol1, self.mol2, lambda m, i: sm.GetRDKFingerprint(m, i, nBits=1024, nBitsPerHash=1))
    for w, r in zip(weights, refWeights):
      self.assertAlmostEqual(w, r, 4)

  @unittest.skipUnless(matplotlib, 'Matplotlib required')
  def testSimilarityMapKWArgs(self):
    # Morgan2 BV
    m1 = Chem.MolFromSmiles('CC[C@](F)(Cl)c1ccccc1')
    m2 = Chem.MolFromSmiles('CC[C@@](F)(Cl)c1ccccc1')
    weights = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetAPFingerprint(m, atomId=i, includeChirality=False))
    for w in weights:
      self.assertAlmostEqual(w, 0.100, 4)
    weights = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetAPFingerprint(m, atomId=i, includeChirality=True))
    for i, w in enumerate(weights):
      if i != 2:
        self.assertAlmostEqual(w, 0.098, 3)
      else:
        self.assertAlmostEqual(w, -0.082, 3)

    weights = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetTTFingerprint(m, atomId=i, includeChirality=False))
    for w in weights:
      self.assertTrue(w > 0.0)
    weights = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetTTFingerprint(m, atomId=i, includeChirality=True))
    for i, w in enumerate(weights):
      if i > 4:
        self.assertTrue(w > 0.0)
      else:
        self.assertTrue(w < 0.0)

    weights = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetMorganFingerprint(m, radius=1, atomId=i, useChirality=False))
    weights2 = sm.GetAtomicWeightsForFingerprint(
      m1, m2, lambda m, i: sm.GetMorganFingerprint(m, radius=1, atomId=i, useChirality=True))
    # testing explicit values here seems silly, just check that the contribution of the
    # chiral center drops:
    self.assertTrue(weights[2] > weights2[2])

  def testSimilarityMapsMolDraw2D(self):
    # nothing really sensible to test here, just make sure things run
    mol = Chem.MolFromSmiles('COc1cccc2cc(C(=O)NCCCCN3CCN(c4cccc5nccnc54)CC3)oc21')
    refmol = Chem.MolFromSmiles('CCCN(CCCCN1CCN(c2ccccc2OC)CC1)Cc1ccc2ccccc2c1')
    d = Draw.MolDraw2DSVG(400, 400)
    d.ClearDrawing()
    _, maxWeight = sm.GetSimilarityMapForFingerprint(
      refmol, mol, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'), draw2d=d)
    d.FinishDrawing()
    with open('similarityMap1_out.svg', 'w+') as outf:
      outf.write(d.GetDrawingText())

    # Github #2904: make sure we can provide our own colormap as a list:
    colors = [(0, 1, 0, 0.5), (1, 1, 1), (0, 0, 1, 0.5)]
    d = Draw.MolDraw2DSVG(400, 400)
    d.ClearDrawing()
    _, maxWeight = sm.GetSimilarityMapForFingerprint(
      refmol, mol, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'), draw2d=d,
      colorMap=colors)
    d.FinishDrawing()
    with open('similarityMap1_out2.svg', 'w+') as outf:
      outf.write(d.GetDrawingText())

    # Github #2904: make sure we can provide our own colormap as a matplotlib colormap:
    try:
      from matplotlib import cm
      d = Draw.MolDraw2DSVG(400, 400)
      d.ClearDrawing()
      _, maxWeight = sm.GetSimilarityMapForFingerprint(
        refmol, mol, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'), draw2d=d,
        colorMap=cm.PiYG)
      d.FinishDrawing()
      with open('similarityMap1_out3.svg', 'w+') as outf:
        outf.write(d.GetDrawingText())
    except ImportError:
      pass

  @unittest.skipUnless(matplotlib, 'Matplotlib required')
  def testGithub4763(self):
    mol = Chem.MolFromSmiles('COc1cccc2cc(C(=O)NCCCCN3CCN(c4cccc5nccnc54)CC3)oc21')
    refmol = Chem.MolFromSmiles('CCCN(CCCCN1CCN(c2ccccc2OC)CC1)Cc1ccc2ccccc2c1')
    d = Draw.MolDraw2DSVG(400, 400)
    d.ClearDrawing()
    _, maxWeight = sm.GetSimilarityMapForFingerprint(
      refmol, mol, lambda m, i: sm.GetMorganFingerprint(m, i, radius=2, fpType='bv'), draw2d=d,
      colorMap="coolwarm")
    d.FinishDrawing()
    svg = d.GetDrawingText()
    with open('github4763.svg', 'w+') as outf:
      outf.write(svg)
    self.assertFalse('fill:#FBFCFB7F' in svg)
    self.assertTrue('fill:#DDDCDB' in svg)


if __name__ == '__main__':
  unittest.main()



import re

splitExpr = re.compile('[\t ]')

from Chem import rdmol


def runit(fName):
  inLines = open(fName, 'r').readlines()
  nFailed = 0
  nPassed = 0
  nTried = 0
  for line in inLines:
    if len(line):
      smi = splitExpr.split(line)[1]
      if smi[-1] == '\n':
        smi = smi[:-1]
      if smi[-1] == '\r':
        smi = smi[:-1]
      nTried += 1
      print('trying: "%s"' % smi)
      m = rdmol.MolFromSmiles(smi)
      nPassed += 1

      print('frags')
      nFrags = max(rdmol.FindMolFrags(m)) + 1
      print('sssr')
      rings = rdmol.FindSSSR(m)
      cyclomat = m.getNumBonds() - m.getNumAtoms() + nFrags
      assert cyclomat == len(rings), 'bad num rings for %s\n\t%s!=%s' % (smi, cyclomat, len(rings))
      print('span')
      nChords = m.getNumBonds() - len(rdmol.FindSpanningTree(m))
      assert len(rings) == nChords, 'bad num chords for %s\n\t%s!=%s' % (smi, nChords, len(rings))
      m = None

  print('%d of %d passed' % (nPassed, nTried))


if __name__ == '__main__':
  import sys
  fName = 'ntp_smiles.txt'
  if len(sys.argv) > 1:
    fName = sys.argv[1]

  runit(fName)



# $Id$
#
#  Copyright (C) 2004  Rational Discovery LLC
#         All Rights Reserved
#
import os
import sys
import unittest

from rdkit import Chem, Geometry, RDConfig
from rdkit.Geometry import Point3D


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


def ptEq(pt1, pt2, tol=1e-4):
  return feq(pt1.x, pt2.x, tol) and feq(pt1.y, pt2.y, tol) and feq(pt1.z, pt2.z, tol)


def addConf(mol):
  conf = Chem.Conformer(mol.GetNumAtoms())
  for i in range(mol.GetNumAtoms()):
    conf.SetAtomPosition(i, (0., 0., 0.))
  mol.AddConformer(conf)
  mb = Chem.MolToMolBlock(mol)
  mb = Chem.MolToMolBlock(mol)


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test0Conformers(self):
    """Test the conformer data structure"""
    mol = Chem.MolFromSmiles("CC")
    conf = Chem.Conformer(2)
    conf.SetAtomPosition(0, (-0.5, 0.0, 0.0))
    conf.SetAtomPosition(1, (1.0, 0.0, 0.0))
    conf.SetId(0)
    cid = mol.AddConformer(conf)

    self.assertTrue(cid == 0)

    conf2 = mol.GetConformer(0)
    self.assertTrue(conf2.GetId() == cid)
    pt1 = conf2.GetAtomPosition(0)
    self.assertTrue(ptEq(pt1, Point3D(-0.5, 0.0, 0.0)))

    pt2 = conf2.GetAtomPosition(1)
    self.assertTrue(ptEq(pt2, Point3D(1.0, 0.0, 0.0)))

    #changing conf should not change conf2 - related to issue 217
    conf.SetAtomPosition(1, Point3D(2.0, 0.0, 0.0))
    pt2 = conf2.GetAtomPosition(1)
    self.assertTrue(feq(pt2[0], 1.0))

    conf = Chem.Conformer(2)
    conf.SetAtomPosition(0, Point3D(-0.5, 0.0, 0.0))
    conf.SetAtomPosition(1, Point3D(1.0, 0.0, 0.0))
    conf.SetId(2)

    cid = mol.AddConformer(conf, 0)
    self.assertTrue(cid == 2)
    conf3 = mol.GetConformer(2)

  def test0AddHds(self):
    mol = Chem.MolFromSmiles("CC")
    conf = Chem.Conformer(1)
    conf.SetAtomPosition(0, Point3D(-0.5, 0.0, 0.0))
    conf.SetAtomPosition(1, Point3D(1.0, 0.0, 0.0))
    cid = mol.AddConformer(conf)

    conf2 = mol.GetConformer()
    self.assertTrue(conf2.GetNumAtoms() == 2)

    nmol = Chem.AddHs(mol, 0, 1)
    conf3 = nmol.GetConformer()
    self.assertTrue(conf3.GetNumAtoms() == 8)
    self.assertTrue(conf2.GetNumAtoms() == 2)

    targetCoords = [[-0.5, 0.0, 0.0], [1.0, 0.0, 0.0], [-0.8667, 0.0, 1.03709],
                    [-0.8667, 0.8981, -0.5185], [-0.8667, -0.8981, -0.5185], [1.3667, 0.0, -1.0371],
                    [1.36667, 0.8981, 0.5185], [1.36667, -0.8981, 0.5185]]

    for i in range(8):
      pt = conf3.GetAtomPosition(i)
      self.assertTrue(ptEq(pt, Point3D(*tuple(targetCoords[i]))))

  def test2Issue217(self):
    smi = 'c1ccccc1'
    m = Chem.MolFromSmiles(smi)
    addConf(m)
    self.assertTrue(m.GetNumConformers() == 1)
    mb2 = Chem.MolToMolBlock(m)

  def test3Exceptions(self):
    smi = 'c1ccccc1'
    m = Chem.MolFromSmiles(smi)
    addConf(m)
    self.assertTrue(m.GetNumConformers() == 1)
    self.assertRaises(ValueError, lambda: m.GetConformer(2))

  def test4ConfTuple(self):
    smi = 'c1ccccc1'
    m = Chem.MolFromSmiles(smi)
    for i in range(10):
      addConf(m)

    confs = m.GetConformers()
    self.assertTrue(len(confs) == 10)

    for conf in confs:
      for i in range(6):
        pt = conf.GetAtomPosition(i)
        self.assertTrue(ptEq(pt, Point3D(0.0, 0.0, 0.0)))

    m.RemoveAllConformers()
    self.assertTrue(m.GetNumConformers() == 0)
    confs = m.GetConformers()
    self.assertEqual(len(confs), 0)

  def test5PositionsArray(self):
    smi = 'c1ccccc1'
    m = Chem.MolFromSmiles(smi)
    addConf(m)

    confs = m.GetConformers()
    self.assertTrue(len(confs) == 1)

    for conf in confs:
      pos = conf.GetPositions()
      self.assertTrue(pos.shape, (6, 3))

    m.RemoveAllConformers()
    self.assertTrue(m.GetNumConformers() == 0)
    confs = m.GetConformers()
    self.assertEqual(len(confs), 0)


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2003-2006  Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""basic unit testing code for the rdkit Boost wrapper

"""
import os
import pickle
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import AllChem


class TestCase(unittest.TestCase):

  def setUp(self):
    self.bigSmiList = [
      "CC1=CC(=O)C=CC1=O",
      "S(SC1=NC2=CC=CC=C2S1)C3=NC4=C(S3)C=CC=C4",
      "OC1=C(Cl)C=C(C=C1[N+]([O-])=O)[N+]([O-])=O",
      "[O-][N+](=O)C1=CNC(=N)S1",
      "NC1=CC2=C(C=C1)C(=O)C3=C(C=CC=C3)C2=O",
      "OC(=O)C1=C(C=CC=C1)C2=C3C=CC(=O)C(=C3OC4=C2C=CC(=C4Br)O)Br",
      "CN(C)C1=C(Cl)C(=O)C2=C(C=CC=C2)C1=O",
      "CC1=C(C2=C(C=C1)C(=O)C3=CC=CC=C3C2=O)[N+]([O-])=O",
      "CC(=NO)C(C)=NO",
      "C1=CC=C(C=C1)P(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC(C)(C)C1=C(O)C=C(C(=C1)O)C(C)(C)C",
      "CC1=NN(C(=O)C1)C2=CC=CC=C2",
      "NC1=CC=NC2=C1C=CC(=C2)Cl",
      "CCCCCC[CH]1CCCCN1",
      "O=CC1=C2C=CC=CC2=CC3=C1C=CC=C3",
      "BrN1C(=O)CCC1=O",
      "CCCCCCCCCCCCCCCC1=C(N)C=CC(=C1)O",
      "C(COC1=C(C=CC=C1)C2=CC=CC=C2)OC3=CC=CC=C3C4=CC=CC=C4",
      "CCCCSCC",
      "CC(=O)NC1=NC2=C(C=C1)C(=CC=N2)O",
      "CC1=C2C=CC(=NC2=NC(=C1)O)N",
      "CCOC(=O)C1=CN=C2N=C(N)C=CC2=C1O",
      "CC1=CC(=NC=C1)N=CC2=CC=CC=C2",
      "C[N+](C)(C)CC1=CC=CC=C1",
      "C[N+](C)(C)C(=O)C1=CC=CC=C1",
      "ICCC(C1=CC=CC=C1)(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC1=CC(=C(C[N+](C)(C)C)C(=C1)C)C",
      "C[C](O)(CC(O)=O)C1=CC=C(C=C1)[N+]([O-])=O",
      "CC1=CC=C(C=C1)C(=O)C2=CC=C(Cl)C=C2",
      "ON=CC1=CC=C(O)C=C1",
      "CC1=CC(=C(N)C(=C1)C)C",
      "CC1=CC=C(C=C1)C(=O)C2=CC=C(C=C2)[N+]([O-])=O",
      "CC(O)(C1=CC=CC=C1)C2=CC=CC=C2",
      "ON=CC1=CC(=CC=C1)[N+]([O-])=O",
      "OC1=C2C=CC(=CC2=NC=C1[N+]([O-])=O)Cl",
      "CC1=CC=CC2=NC=C(C)C(=C12)Cl",
      "CCC(CC)([CH](OC(N)=O)C1=CC=CC=C1)C2=CC=CC=C2",
      "ON=C(CC1=CC=CC=C1)[CH](C#N)C2=CC=CC=C2",
      "O[CH](CC1=CC=CC=C1)C2=CC=CC=C2",
      "COC1=CC=C(CC2=CC=C(OC)C=C2)C=C1",
      "CN(C)[CH](C1=CC=CC=C1)C2=C(C)C=CC=C2",
      "COC1=CC(=C(N)C(=C1)[N+]([O-])=O)[N+]([O-])=O",
      "NN=C(C1=CC=CC=C1)C2=CC=CC=C2",
      "COC1=CC=C(C=C1)C=NO",
      "C1=CC=C(C=C1)C(N=C(C2=CC=CC=C2)C3=CC=CC=C3)C4=CC=CC=C4",
      "C1=CC=C(C=C1)N=C(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC1=C(C2=CC=CC=C2)C(=C3C=CC=CC3=N1)O",
      "CCC1=[O+][Cu]2([O+]=C(CC)C1)[O+]=C(CC)CC(=[O+]2)CC",
      "OC(=O)[CH](CC1=CC=CC=C1)C2=CC=CC=C2",
      "CCC1=C(N)C=C(C)N=C1",
    ]

  def test1(self):
    """ basic building stuff """
    m = Chem.MolFromSmiles('COC(=O)O')

    a1 = m.GetAtomWithIdx(1)
    assert a1.GetAtomicNum() == 8
    assert m.GetAtomWithIdx(2).GetAtomicNum() == 6
    b1 = m.GetBondWithIdx(1)
    assert b1.GetBondType() == Chem.BondType.SINGLE
    assert m.GetBondWithIdx(2).GetBondType() == Chem.BondType.DOUBLE
    assert m.GetBondBetweenAtoms(0, 1).GetBondType() == Chem.BondType.SINGLE
    assert m.GetBondBetweenAtoms(2, 3).GetBondType() == Chem.BondType.DOUBLE

  def test2(self):
    """ editing/persistence basics """
    m = Chem.MolFromSmiles('COC(=C)O')

    a1 = m.GetAtomWithIdx(3)
    assert a1.GetAtomicNum() == 6, 'bad atom order'
    a1.SetAtomicNum(7)
    assert a1.GetAtomicNum() == 7, 'bad atom order'
    assert m.GetAtomWithIdx(3).GetAtomicNum() == 7, 'atom order not stored'

  def test3(self):
    """ SMARTS basics """
    m = Chem.MolFromSmiles('COC(=O)O')
    p = Chem.MolFromSmarts('CO')
    assert m.HasSubstructMatch(p)
    p2 = Chem.MolFromSmarts('CS')
    assert not m.HasSubstructMatch(p2)

    #assert p.GetSMARTS()=='CO','bad getsmarts'
    assert p.GetNumAtoms() == 2
    assert p.GetNumBonds() == 1

    assert m.HasSubstructMatch(p)
    matches = m.GetSubstructMatches(p)
    assert len(matches) == 3
    match = matches[0]
    assert len(match) == 2, 'bad match length'
    matches = m.GetSubstructMatches(p, 0)
    assert len(matches) == 3
    match = matches[0]
    assert len(match) == 2, 'bad match length'

    p = Chem.MolFromSmarts('COC')
    assert m.HasSubstructMatch(p)
    matches = m.GetSubstructMatches(p)
    assert len(matches) == 1
    matches = m.GetSubstructMatches(p, 0)
    assert len(matches) == 2

  def test4Pkl2(self):
    """ further pickle tests """
    smis = self.bigSmiList

    for smi in smis:
      m = Chem.MolFromSmiles(smi)
      newM1 = pickle.loads(pickle.dumps(m))
      newM2 = pickle.loads(pickle.dumps(newM1))
      oldSmi = Chem.MolToSmiles(newM1)
      newSmi = Chem.MolToSmiles(newM2)
      assert newM1.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert newM2.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert oldSmi == newSmi, 'string compare failed: %s != %s' % (oldSmi, newSmi)

  def test5Data(self):
    """ testing Get/Set/HasData """
    m = Chem.MolFromSmiles('CCOC')
    try:
      m.SetProp('foo', '3')
    except Exception:
      ok = 0
    else:
      ok = 1
    assert ok

    try:
      v = m.GetProp('foo')
    except Exception:
      ok = 0
    else:
      ok = 1
    assert ok
    assert v == '3'

    try:
      v = m.GetProp('monkey')
    except KeyError:
      ok = 1
    except Exception:
      ok = 0
    else:
      ok = 0
    assert ok

  def testIssue399(self):
    m = Chem.MolFromSmiles('C[C@H]1CO1')
    AllChem.Compute2DCoords(m)
    Chem.WedgeMolBonds(m, m.GetConformer())
    self.assertTrue(m.GetBondWithIdx(0).GetBondDir() == Chem.rdchem.BondDir.BEGINDASH)
    self.assertTrue(m.GetBondWithIdx(1).GetBondDir() == Chem.rdchem.BondDir.NONE)
    self.assertTrue(m.GetBondWithIdx(2).GetBondDir() == Chem.rdchem.BondDir.NONE)
    self.assertTrue(m.GetBondWithIdx(3).GetBondDir() == Chem.rdchem.BondDir.NONE)


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2001-2008  greg Landrum
#
""" unit testing code for the Doctests

"""
import doctest
import unittest
from io import StringIO

from rdkit.ML.Data import DataUtils, SplitData
from rdkit.TestRunner import redirect_stdout


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(SplitData, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(DataUtils, optionflags=doctest.ELLIPSIS))
  return tests


class TestCaseSplitData(unittest.TestCase):

  def test_exceptions(self):
    self.assertRaises(ValueError, SplitData.SplitIndices, 10, -0.1)
    self.assertRaises(ValueError, SplitData.SplitIndices, 10, 1.1)

    f = StringIO()
    with redirect_stdout(f):
      SplitData.SplitIndices(10, 0.5, replacement=True, silent=False)
    s = f.getvalue()
    self.assertIn('Training', s)
    self.assertIn('hold-out', s)

  def test_SplitData(self):
    self.assertRaises(ValueError, SplitData.SplitDataSet, None, -1.1)
    self.assertRaises(ValueError, SplitData.SplitDataSet, None, 1.1)

    data = list(range(10))
    DataUtils.InitRandomNumbers((23, 42))
    f = StringIO()
    with redirect_stdout(f):
      result = SplitData.SplitDataSet(data, 0.5)
    self.assertEqual(set(result[0]).intersection(result[1]), set())
    self.assertEqual(len(result[0]), 5)
    s = f.getvalue()
    self.assertIn('Training', s)
    self.assertIn('hold-out', s)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import numpy

from rdkit import RDRandom
from rdkit.SimDivFilters import rdSimDivPickers as rdsimdiv

RDRandom.seed(23)

pkr = rdsimdiv.MaxMinPicker()

n = 1000
m = 80
dataPts = []
for i in range(n):
  pt = numpy.zeros(2, 'd')
  pt[0] = 10. * RDRandom.random()
  pt[1] = 10. * RDRandom.random()
  dataPts.append(pt)

# compute the distance matrix
distMat = numpy.zeros(n * (n - 1) / 2, 'd')
for i in range(n - 1):
  itab = n * i - ((i + 1) * (i + 2)) / 2
  pt1 = dataPts[i]
  for j in range(i + 1, n):
    id = itab + j
    pt2 = dataPts[j]
    diff = pt2 - pt1

    dist = numpy.sqrt(numpy.dot(diff, diff))
    distMat[id] = dist

  # now do the picking
res = pkr.Pick(distMat, n, m)

print("Results:")
for k in res:
  print(dataPts[k][0], dataPts[k][1])



#
#  Copyright (C) 2020 Greg Landrum and T5 Informatics GmbH
#   @@ All Rights Reserved @@
#
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.

import unittest

#
from rdkit import Chem
from rdkit.Chem import rdAbbreviations


class TestCase(unittest.TestCase):

  def setUp(self):
    self.defaultAbbrevs = rdAbbreviations.GetDefaultAbbreviations()
    self.defaultLinkers = rdAbbreviations.GetDefaultLinkers()
    self.customLinkers = rdAbbreviations.ParseLinkers('''PEG3  *OCCOCCOCC* PEG3
Pent  *CCCCC*
Cy   *C1CCC(*)CC1  Cy''')
    self.parseMapping = lambda mapping: tuple(map(int, mapping.strip('[],').split(',')))

  def testParsingAbbrevs(self):
    defn = '''CO2Et    C(=O)OCC
COOEt    C(=O)OCC
OiBu     OCC(C)C
tBu      C(C)(C)C'''
    abbrevs = rdAbbreviations.ParseAbbreviations(defn)
    m = Chem.MolFromSmiles('CCC(=O)OCC')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, abbrevs, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), '*CC |$CO2Et;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1))

  def testCondense(self):
    m = Chem.MolFromSmiles('FC(F)(F)CC(=O)O')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.defaultAbbrevs, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), '*C* |$CF3;;CO2H$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (1, 4, 5))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (3, 4))

    m = Chem.MolFromSmiles('CCC(F)(F)F')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.defaultAbbrevs)
    self.assertEqual(Chem.MolToCXSmiles(nm), '*C(F)(F)F |$Et;;;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (1, 2, 3, 4, 5))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (1, 2, 3, 4))

    # make sure we don't mess up chirality
    m = Chem.MolFromSmiles('FC(F)(F)[C@](Cl)(F)I')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.defaultAbbrevs, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), '*[C@@](F)(Cl)I |$CF3;;;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (1, 4, 5, 6, 7))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (3, 4, 5, 6))

  def testLabel(self):
    m = Chem.MolFromSmiles('CC(C)CC(F)(F)F')
    nm = rdAbbreviations.LabelMolAbbreviations(m, self.defaultAbbrevs, maxCoverage=1.0)
    sgs = Chem.GetMolSubstanceGroups(nm)
    self.assertEqual(len(sgs), 2)
    self.assertEqual(sgs[0].GetProp('TYPE'), "SUP")
    self.assertEqual(sgs[0].GetProp('LABEL'), "iPr")
    self.assertEqual(list(sgs[0].GetAtoms()), [1, 0, 2])
    self.assertEqual(list(sgs[0].GetBonds()), [2])
    aps = sgs[0].GetAttachPoints()
    self.assertEqual(len(aps), 1)
    self.assertEqual(aps[0].aIdx, 1)
    self.assertEqual(aps[0].lvIdx, 3)

    self.assertEqual(sgs[1].GetProp('TYPE'), "SUP")
    self.assertEqual(sgs[1].GetProp('LABEL'), "CF3")
    self.assertEqual(list(sgs[1].GetAtoms()), [4, 5, 6, 7])
    self.assertEqual(list(sgs[1].GetBonds()), [3])
    aps = sgs[1].GetAttachPoints()
    self.assertEqual(len(aps), 1)
    self.assertEqual(aps[0].aIdx, 4)
    self.assertEqual(aps[0].lvIdx, 3)

  def testCondenseLinkers(self):
    m = Chem.MolFromSmiles('FCOCCOCCOCCCCCCCCCCl')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.defaultLinkers, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), 'FC**Cl |$;;PEG3;Hept;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2, 11, 18))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1, 10, 17))

    m = Chem.MolFromSmiles('COC1CCC(C)CC1')
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.customLinkers, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), 'C*OC |$;Cy;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2, 6))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1, 5))

  def testAbbreviationsAndLinkers(self):
    m = Chem.MolFromSmiles('COC1CCC(C)CC1')
    # wouldn't normally do this in this order:
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.defaultAbbrevs, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), '*C1CCC(C)CC1 |$OMe;;;;;;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (1, 2, 3, 4, 5, 6, 7, 8))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (1, 2, 3, 4, 5, 6, 7, 8))
    nm = rdAbbreviations.CondenseMolAbbreviations(nm, self.customLinkers, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), '**C |$OMe;Cy;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (1, 2, 6))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (1, 5))

    # This is a more logical order
    nm = rdAbbreviations.CondenseMolAbbreviations(m, self.customLinkers, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), 'C*OC |$;Cy;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2, 6))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1, 5))
    nm = rdAbbreviations.CondenseMolAbbreviations(nm, self.defaultAbbrevs, maxCoverage=1.0)
    self.assertEqual(Chem.MolToCXSmiles(nm), 'C*OC |$;Cy;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2, 6))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1, 5))

  def testAbbreviationsSubstanceGroups(self):
    m = Chem.MolFromMolBlock('''
  Mrv2014 09152006492D          

  0  0  0     0  0            999 V3000
M  V30 BEGIN CTAB
M  V30 COUNTS 7 7 1 0 0
M  V30 BEGIN ATOM
M  V30 1 C 5.25 -5.9858 0 0
M  V30 2 C 4.48 -7.3196 0 0
M  V30 3 C 6.02 -7.3196 0 0
M  V30 4 F 8.6873 -8.8596 0 0
M  V30 5 C 7.3537 -8.0896 0 0
M  V30 6 F 6.02 -8.8596 0 0
M  V30 7 F 7.3537 -6.5496 0 0
M  V30 END ATOM
M  V30 BEGIN BOND
M  V30 1 1 1 2
M  V30 2 1 3 1
M  V30 3 1 2 3
M  V30 4 1 3 5
M  V30 5 1 4 5
M  V30 6 1 5 6
M  V30 7 1 5 7
M  V30 END BOND
M  V30 BEGIN SGROUP
M  V30 1 SUP 0 ATOMS=(4 4 5 6 7) SAP=(3 5 3 1) XBONDS=(1 4) LABEL=CF3
M  V30 END SGROUP
M  V30 END CTAB
M  END''')
    nm = rdAbbreviations.CondenseAbbreviationSubstanceGroups(m)
    nm.RemoveAllConformers()  # avoid coords in CXSMILES
    self.assertEqual(Chem.MolToCXSmiles(nm), '*C1CC1 |$CF3;;;$|')
    self.assertTrue(nm.HasProp('_origAtomMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origAtomMapping')), (0, 1, 2, 4))
    self.assertTrue(nm.HasProp('_origBondMapping'))
    self.assertEqual(self.parseMapping(nm.GetProp('_origBondMapping')), (0, 1, 2, 3))

  def testGithub3692(self):
    defaults = rdAbbreviations.GetDefaultAbbreviations()
    self.assertIsNotNone(defaults[0].mol)
    lbls = [x.label for x in defaults]
    self.assertIn('CO2Et', lbls)
    idx = lbls.index('CO2Et')
    self.assertEqual(Chem.MolToSmiles(defaults[idx].mol), '*C(=O)OCC')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



"""
unit tests for the MolStandardize module
tests include:
reorder_tautomers
"""

import unittest

from rdkit import Chem
from rdkit.Chem import MolStandardize
from rdkit.Chem.MolStandardize import rdMolStandardize


class TestCase(unittest.TestCase):

  def testBasic(self):
    m = Chem.MolFromSmiles('Oc1c(cccc3)c3nc2ccncc12')
    enumerator = rdMolStandardize.TautomerEnumerator()
    canon = enumerator.Canonicalize(m)
    reord = MolStandardize.ReorderTautomers(m)[0]
    canonSmile = Chem.MolToSmiles(canon)
    reordSmile = Chem.MolToSmiles(reord)
    self.assertEqual(canonSmile, reordSmile)

  def testLength(self):
    m = Chem.MolFromSmiles('Oc1c(cccc3)c3nc2ccncc12')
    enumerator = rdMolStandardize.TautomerEnumerator()
    tauts = enumerator.Enumerate(m)
    reordtauts = MolStandardize.ReorderTautomers(m)
    self.assertEqual(len(reordtauts), len(tauts))



# $Id$
#
#  Copyright (C) 2004-2006 Rational Discovery LLC
#
#     @@  All Rights Reserved  @@
#

import copy
import math
import os
import sys
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import (ChemicalForceFields, rdDistGeom, rdMolAlign, rdMolDescriptors,
                        rdMolTransforms)


def lstFeq(l1, l2, tol=1.e-4):
  if (len(list(l1)) != len(list(l2))):
    return 0
  for i in range(len(list(l1))):
    if not feq(l1[i], l2[i], tol):
      return 0
  return 1


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1Basic(self):
    file1 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir.mol')
    file2 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir_conf.mol')

    mol1 = Chem.MolFromMolFile(file1)
    mol2 = Chem.MolFromMolFile(file2)

    rmsd = rdMolAlign.AlignMol(mol2, mol1)
    self.assertTrue(feq(rmsd, 0.6578))

    file3 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir_trans.mol')
    mol3 = Chem.MolFromMolFile(file3)
    conf2 = mol2.GetConformer()
    conf3 = mol3.GetConformer()

    for i in range(mol2.GetNumAtoms()):
      self.assertTrue(lstFeq(conf2.GetAtomPosition(i), conf3.GetAtomPosition(i)))

    rmsd, trans = rdMolAlign.GetAlignmentTransform(mol2, mol1)
    self.assertAlmostEqual(rmsd, 0.6579, 4)

  def test2AtomMap(self):
    atomMap = ((18, 27), (13, 23), (21, 14), (24, 7), (9, 19), (16, 30))
    file1 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir.mol')
    file2 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir_conf.mol')

    mol1 = Chem.MolFromMolFile(file1)
    mol2 = Chem.MolFromMolFile(file2)
    rmsd = rdMolAlign.AlignMol(mol2, mol1, 0, 0, atomMap)
    self.assertAlmostEqual(rmsd, 0.8525, 4)

  def test3Weights(self):
    atomMap = ((18, 27), (13, 23), (21, 14), (24, 7), (9, 19), (16, 30))
    file1 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir.mol')
    file2 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         '1oir_conf.mol')

    mol1 = Chem.MolFromMolFile(file1)
    mol2 = Chem.MolFromMolFile(file2)
    wts = (1.0, 1.0, 1.0, 1.0, 1.0, 2.0)
    rmsd = rdMolAlign.AlignMol(mol2, mol1, 0, 0, atomMap, wts)
    self.assertAlmostEqual(rmsd, 0.9513, 4)

  def test4AlignConfs(self):
    mol = Chem.MolFromSmiles('C1CC1CNc(n2)nc(C)cc2Nc(cc34)ccc3[nH]nc4')

    cids = rdDistGeom.EmbedMultipleConfs(mol, 10, 30, 100)
    #writer = Chem.SDWriter('mol_899.sdf')

    for cid in cids:
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol, confId=cid)
      ff.Initialize()
      more = 1
      while more:
        more = ff.Minimize()
      # FIX: this should not be necessary but somehow more comes out to be 0
      # even with the structure still being crappy
      ff.Minimize()
    aids = [12, 13, 14, 15, 16, 17, 18]
    rdMolAlign.AlignMolConformers(mol, aids)

    # now test that the atom location of these atom are consistent
    confs = mol.GetConformers()
    for aid in aids:
      mpos = 0
      for i, conf in enumerate(confs):
        if (i == 0):
          mpos = list(conf.GetAtomPosition(aid))
          continue
        else:
          pos = list(conf.GetAtomPosition(aid))

          self.assertTrue(lstFeq(mpos, pos, .5))

    # now test that we can get a list of RMS values
    rmsvals = []
    rdMolAlign.AlignMolConformers(mol, aids, RMSlist=rmsvals)
    self.assertTrue((len(rmsvals) == mol.GetNumConformers() - 1))

    # make sure something sensible happens if we provide a stupid
    # argument:
    rmsvals = 4
    self.assertRaises(AttributeError, rdMolAlign.AlignMolConformers, mol, atomIds=aids,
                      RMSlist=rmsvals)

  def test5MMFFO3A(self):
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    # alignedSdf = os.path.join(RDConfig.RDBaseDir,'Code','GraphMol',
    #                           'MolAlign', 'test_data', 'ref_e2_pyMMFFO3A.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    # molW = Chem.SDWriter(alignedSdf)
    refNum = 48
    refMol = molS[refNum]
    cumScore = 0.0
    cumMsd = 0.0
    refPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(refMol)
    for prbMol in molS:
      prbPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(prbMol)
      pyO3A = rdMolAlign.GetO3A(prbMol, refMol, prbPyMP, refPyMP)
      cumScore += pyO3A.Score()
      rmsd = pyO3A.Align()
      cumMsd += rmsd * rmsd
      # molW.write(prbMol)
    cumMsd /= len(molS)
    self.assertAlmostEqual(cumScore, 6942, 0)
    self.assertAlmostEqual(math.sqrt(cumMsd), .345, 3)

  def test6MMFFO3A(self):
    " now test where the mmff parameters are generated on call "
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    refNum = 48
    refMol = molS[refNum]
    cumScore = 0.0
    cumMsd = 0.0
    for prbMol in molS:
      pyO3A = rdMolAlign.GetO3A(prbMol, refMol)
      cumScore += pyO3A.Score()
      rmsd = pyO3A.Align()
      cumMsd += rmsd * rmsd
    cumMsd /= len(molS)
    self.assertAlmostEqual(cumScore, 6942, 0)
    self.assertAlmostEqual(math.sqrt(cumMsd), .345, 3)

  def test7MMFFO3A(self):
    " make sure we generate an error if parameters are missing (github issue 158) "

    m1 = Chem.MolFromSmiles('c1ccccc1Cl')
    rdDistGeom.EmbedMolecule(m1)
    m2 = Chem.MolFromSmiles('c1ccccc1B(O)O')
    rdDistGeom.EmbedMolecule(m1)

    self.assertRaises(ValueError, lambda: rdMolAlign.GetO3A(m1, m2))
    self.assertRaises(ValueError, lambda: rdMolAlign.GetO3A(m2, m1))

  def test8MMFFO3A(self):
    " test MMFFO3A with constraints "

    #we superimpose two identical coplanar 4-phenylpyridines:
    #1) the usual way
    #2) forcing the pyridine nitrogen to match with the para
    #   carbon of the phenyl ring
    m = Chem.MolFromSmiles('n1ccc(cc1)-c1ccccc1')
    m1 = Chem.AddHs(m)
    rdDistGeom.EmbedMolecule(m1)
    mp = ChemicalForceFields.MMFFGetMoleculeProperties(m1)
    ff = ChemicalForceFields.MMFFGetMoleculeForceField(m1, mp)
    ff.Minimize()
    sub1 = m1.GetSubstructMatch(Chem.MolFromSmarts('nccc-cccc'))
    nIdx = sub1[0]
    cIdx = sub1[-1]
    dihe = sub1[2:6]
    rdMolTransforms.SetDihedralDeg(m1.GetConformer(), dihe[0], dihe[1], dihe[2], dihe[3], 0)
    m2 = copy.copy(m1)
    rdMolAlign.RandomTransform(m2)
    m3 = copy.copy(m2)
    pyO3A = rdMolAlign.GetO3A(m2, m1)
    pyO3A.Align()
    d = m2.GetConformer().GetAtomPosition(cIdx). \
      Distance(m1.GetConformer().GetAtomPosition(cIdx))
    self.assertAlmostEqual(d, 0, 0)
    pyO3A = rdMolAlign.GetO3A(m3, m1, constraintMap=[[cIdx, nIdx]])
    pyO3A.Align()
    d = m3.GetConformer().GetAtomPosition(cIdx). \
      Distance(m1.GetConformer().GetAtomPosition(cIdx))
    self.assertAlmostEqual(d, 7, 0)
    #alignedSdf = os.path.join(RDConfig.RDBaseDir,'Code','GraphMol',
    #                          'MolAlign', 'test_data',
    #                          '4-phenylpyridines_MMFFO3A.sdf')
    #sdW = Chem.SDWriter(alignedSdf)
    #sdW.write(m1)
    #sdW.write(m2)
    #sdW.write(m3)
    #sdW.close()

  def test9MMFFO3A(self):
    " test MMFFO3A with variable weight constraints followed by local-only optimization "

    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    # alignedSdf = os.path.join(RDConfig.RDBaseDir,'Code','GraphMol',
    #                           'MolAlign', 'test_data', 'localonly.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    refNum = 23
    prbNum = 32
    refMol = molS[refNum]
    prbMol = molS[prbNum]
    refPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(refMol)
    prbPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(prbMol)
    refSIdx = refMol.GetSubstructMatch(Chem.MolFromSmarts('S'))[0]
    prbOIdx = prbMol.GetSubstructMatch(Chem.MolFromSmarts('O'))[0]
    # molW = Chem.SDWriter(alignedSdf)
    # molW.write(refMol)
    weights = [10.0, 100.0]
    distOS = [3.2, 0.3]
    for i in [0, 1]:
      pyO3A = rdMolAlign.GetO3A(prbMol, refMol, prbPyMP, refPyMP,
                                constraintMap=[[prbOIdx, refSIdx]], constraintWeights=[weights[i]])
      pyO3A.Align()
      # molW.write(prbMol)
      pyO3A = rdMolAlign.GetO3A(prbMol, refMol, prbPyMP, refPyMP, options=4)
      pyO3A.Align()
      # molW.write(prbMol)
      d = prbMol.GetConformer().GetAtomPosition(prbOIdx). \
        Distance(refMol.GetConformer().GetAtomPosition(refSIdx))
      self.assertAlmostEqual(d, distOS[i], 1)
    # molW.close()

  def test10CrippenO3A(self):
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    alignedSdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                              'ref_e2_pyCrippenO3A.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    molW = Chem.SDWriter(alignedSdf)
    refNum = 48
    refMol = molS[refNum]
    cumScore = 0.0
    cumMsd = 0.0
    refList = rdMolDescriptors._CalcCrippenContribs(refMol, True)
    for prbMol in molS:
      prbList = rdMolDescriptors._CalcCrippenContribs(prbMol, True)
      pyO3A = rdMolAlign.GetCrippenO3A(prbMol, refMol, prbList, refList)
      cumScore += pyO3A.Score()
      rmsd = pyO3A.Align()
      cumMsd += rmsd * rmsd
      molW.write(prbMol)
    cumMsd /= len(molS)
    self.assertAlmostEqual(cumScore, 4918, 0)
    self.assertAlmostEqual(math.sqrt(cumMsd), .304, 3)

  def test11CrippenO3A(self):
    " now test where the Crippen parameters are generated on call "
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    refNum = 48
    refMol = molS[refNum]
    cumScore = 0.0
    cumMsd = 0.0
    for prbMol in molS:
      pyO3A = rdMolAlign.GetCrippenO3A(prbMol, refMol)
      cumScore += pyO3A.Score()
      rmsd = pyO3A.Trans()[0]
      cumMsd += rmsd * rmsd
    cumMsd /= len(molS)
    self.assertAlmostEqual(cumScore, 4918, 0)
    self.assertAlmostEqual(math.sqrt(cumMsd), .304, 3)

  def test12CrippenO3A(self):
    " test CrippenO3A with constraints "

    #we superimpose two identical coplanar 4-phenylpyridines:
    #1) the usual way
    #2) forcing the pyridine nitrogen to match with the para
    #   carbon of the phenyl ring
    m = Chem.MolFromSmiles('n1ccc(cc1)-c1ccccc1')
    m1 = Chem.AddHs(m)
    rdDistGeom.EmbedMolecule(m1)
    mp = ChemicalForceFields.MMFFGetMoleculeProperties(m1)
    ff = ChemicalForceFields.MMFFGetMoleculeForceField(m1, mp)
    ff.Minimize()
    sub1 = m1.GetSubstructMatch(Chem.MolFromSmarts('nccc-cccc'))
    nIdx = sub1[0]
    cIdx = sub1[-1]
    dihe = sub1[2:6]
    rdMolTransforms.SetDihedralDeg(m1.GetConformer(), dihe[0], dihe[1], dihe[2], dihe[3], 0)
    m2 = copy.copy(m1)
    rdMolAlign.RandomTransform(m2)
    m3 = copy.copy(m2)
    pyO3A = rdMolAlign.GetCrippenO3A(m2, m1)
    pyO3A.Align()
    d = m2.GetConformer().GetAtomPosition(cIdx). \
      Distance(m1.GetConformer().GetAtomPosition(cIdx))
    self.assertAlmostEqual(d, 0, 0)
    pyO3A = rdMolAlign.GetCrippenO3A(m3, m1, constraintMap=[[cIdx, nIdx]])
    pyO3A.Align()
    d = m3.GetConformer().GetAtomPosition(cIdx). \
      Distance(m1.GetConformer().GetAtomPosition(cIdx))
    self.assertAlmostEqual(d, 7, 0)
    #alignedSdf = os.path.join(RDConfig.RDBaseDir,'Code','GraphMol',
    #                          'MolAlign', 'test_data',
    #                          '4-phenylpyridines_CrippenO3A.sdf')
    #sdW = Chem.SDWriter(alignedSdf)
    #sdW.write(m1)
    #sdW.write(m2)
    #sdW.write(m3)
    #sdW.close()

  def test13CrippenO3A(self):
    " test CrippenO3A with variable weight constraints followed by local-only optimization "

    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    # alignedSdf = os.path.join(RDConfig.RDBaseDir,'Code','GraphMol',
    #                           'MolAlign', 'test_data', 'localonly.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    refNum = 23
    prbNum = 32
    refMol = molS[refNum]
    prbMol = molS[prbNum]
    refPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(refMol)
    prbPyMP = ChemicalForceFields.MMFFGetMoleculeProperties(prbMol)
    refSIdx = refMol.GetSubstructMatch(Chem.MolFromSmarts('S'))[0]
    prbOIdx = prbMol.GetSubstructMatch(Chem.MolFromSmarts('O'))[0]
    # molW = Chem.SDWriter(alignedSdf)
    # molW.write(refMol)
    weights = [0.1, 100.0]
    distOS = [2.7, 0.4]
    for i in [0, 1]:
      pyO3A = rdMolAlign.GetCrippenO3A(prbMol, refMol, constraintMap=[[prbOIdx, refSIdx]],
                                       constraintWeights=[weights[i]])
      pyO3A.Align()
      # molW.write(prbMol)
      pyO3A = rdMolAlign.GetCrippenO3A(prbMol, refMol, options=4)
      pyO3A.Align()
      # molW.write(prbMol)
      d = prbMol.GetConformer().GetAtomPosition(prbOIdx). \
        Distance(refMol.GetConformer().GetAtomPosition(refSIdx))
      self.assertAlmostEqual(d, distOS[i], 1)
    # molW.close()

  def test14Github385(self):
    """ test github issue 385:
        O3A code generating incorrect results for multiconformer molecules
      """

    def _multiConfFromSmiles(smiles, nConfs=10, maxIters=500):
      """Adds hydrogens to molecule and optimises a chosen number of conformers.  Returns the optimised RDKit mol."""
      idea = Chem.MolFromSmiles(smiles)
      idea = Chem.AddHs(idea)
      confs = rdDistGeom.EmbedMultipleConfs(idea, nConfs)

      for conf in confs:
        opt = ChemicalForceFields.MMFFOptimizeMolecule(idea, confId=conf, maxIters=maxIters)
      return idea

    def _confsToAlignedMolsList(multiConfMol):
      """Input is a multiconformer RDKit mol.  Output is an aligned set of conformers as a list of RDKit mols."""
      rdMolAlign.AlignMolConformers(multiConfMol)
      ms = []
      cids = [x.GetId() for x in multiConfMol.GetConformers()]
      for cid in cids:
        newmol = Chem.Mol(multiConfMol)
        for ocid in cids:
          if ocid == cid:
            continue
          newmol.RemoveConformer(ocid)
        ms.append(newmol)
      return ms

    reference = Chem.MolFromSmiles("c1ccccc1N2CCC(NS(=O)(=O)C(F)(F)F)CC2")
    reference = Chem.AddHs(reference)
    rdDistGeom.EmbedMolecule(reference)
    idea1 = _multiConfFromSmiles("c1ccccc1C2CCCCC2", 10)

    idea1_mols = _confsToAlignedMolsList(idea1)
    cids = [x.GetId() for x in idea1.GetConformers()]

    refParams = ChemicalForceFields.MMFFGetMoleculeProperties(reference)
    prbParams = ChemicalForceFields.MMFFGetMoleculeProperties(idea1)

    for i in range(len(cids)):
      o3a1 = rdMolAlign.GetO3A(idea1_mols[i], reference, prbParams, refParams)
      score1 = o3a1.Score()

      o3a2 = rdMolAlign.GetO3A(idea1, reference, prbParams, refParams, prbCid=cids[i])
      score2 = o3a2.Score()
      self.assertAlmostEqual(score1, score2, 3)

  def test15MultiConfs(self):
    " test multi-conf alignment "
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'ref_e2.sdf')
    suppl = Chem.SDMolSupplier(sdf, removeHs=False)
    refMol = suppl[13]
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'probe_mol.sdf')
    prbSuppl = Chem.SDMolSupplier(sdf, removeHs=False)
    tms = [x for x in prbSuppl]
    prbMol = tms[0]
    for tm in tms[1:]:
      prbMol.AddConformer(tm.GetConformer(), True)
    self.assertEqual(prbMol.GetNumConformers(), 50)

    refParams = ChemicalForceFields.MMFFGetMoleculeProperties(refMol)
    prbParams = ChemicalForceFields.MMFFGetMoleculeProperties(prbMol)
    cp = Chem.Mol(prbMol)
    o3s = rdMolAlign.GetO3AForProbeConfs(cp, refMol, 1, prbParams, refParams)
    for i in range(prbMol.GetNumConformers()):
      cp2 = Chem.Mol(prbMol)
      o3 = rdMolAlign.GetO3A(cp2, refMol, prbParams, refParams, prbCid=i)
      self.assertAlmostEqual(o3s[i].Align(), o3.Align(), 6)
      self.assertAlmostEqual(o3s[i].Score(), o3.Score(), 6)

    cp = Chem.Mol(prbMol)
    o3s = rdMolAlign.GetCrippenO3AForProbeConfs(cp, refMol)
    for i in range(prbMol.GetNumConformers()):
      cp2 = Chem.Mol(prbMol)
      o3 = rdMolAlign.GetCrippenO3A(cp2, refMol, prbCid=i)
      self.assertAlmostEqual(o3s[i].Align(), o3.Align(), 6)
      self.assertAlmostEqual(o3s[i].Score(), o3.Score(), 6)

  def test16MultithreadBug(self):
    " test multi-conf alignment "
    nConfs = 10
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'bzr_data.sdf')
    bzr_ms_o = [x for x in Chem.SDMolSupplier(sdf, removeHs=False)]
    bzr_ms = [Chem.Mol(x) for x in bzr_ms_o]
    for m in bzr_ms:
      c = m.GetConformer()
      while m.GetNumConformers() < nConfs:
        cc = Chem.Conformer(c)
        m.AddConformer(cc, assignId=True)

    #refParams = ChemicalForceFields.MMFFGetMoleculeProperties(bzr_ms_o[0])

    for i, m in enumerate(bzr_ms):
      #prbParams = ChemicalForceFields.MMFFGetMoleculeProperties(m)
      algs = rdMolAlign.GetO3AForProbeConfs(
        m,
        bzr_ms_o[0],
        numThreads=4  #,prbPyMMFFMolProperties=prbParams,
        #refPyMMFFMolProperties=refParams
      )
      self.assertEqual(len(algs), nConfs)

  def test17GetBestRMS(self):
    sdf = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                       'probe_mol.sdf')
    molS = Chem.SDMolSupplier(sdf, True, False)
    prb = molS[1]
    prbCopy1 = Chem.Mol(prb)
    prbCopy2 = Chem.Mol(prb)
    prbCopy3 = Chem.Mol(prb)
    ref = molS[2]

    rmsdInPlace = rdMolAlign.CalcRMS(prbCopy1, ref)
    self.assertAlmostEqual(rmsdInPlace, 2.6026, 3)
    # AlignMol() would return this for the rms: 2.50561
    # But the best rms is: 2.43449
    rmsd = rdMolAlign.GetBestRMS(prb, ref)
    self.assertAlmostEqual(rmsd, 2.43449, 3)
    rmsdCopy, bestTrans, bestMatch = rdMolAlign.GetBestAlignmentTransform(prbCopy1, ref)
    self.assertEqual(str(type(bestTrans)), "<class 'numpy.ndarray'>")
    self.assertAlmostEqual(rmsd, rmsdCopy, 3)
    self.assertEqual(len(bestMatch), ref.GetNumAtoms())

    params = Chem.SmilesParserParams()
    params.removeHs = False
    scaffold = Chem.MolFromSmiles(
      "N1C([H])([H])C([H])([H])C([H])([H])[N+]([H])([H])C([H])([H])C1([H])[H]", params)
    scaffoldMatch = ref.GetSubstructMatch(scaffold)
    scaffoldIndicesBitSet = [0] * ref.GetNumAtoms()
    for idx in scaffoldMatch:
      scaffoldIndicesBitSet[idx] = 1
    matches = tuple(
      tuple(enumerate(match)) for match in ref.GetSubstructMatches(prb, uniquify=False))
    self.assertGreater(len(matches), 0)
    matchesPruned = tuple(
      tuple(filter(lambda tup: scaffoldIndicesBitSet[tup[1]], match)) for match in matches)
    rmsdInPlace = rdMolAlign.CalcRMS(prbCopy2, ref, map=matchesPruned)
    self.assertAlmostEqual(rmsdInPlace, 2.5672, 3)
    rmsd = rdMolAlign.GetBestRMS(prb, ref, map=matchesPruned)
    self.assertAlmostEqual(rmsd, 1.14329, 3)
    rmsdCopy, bestTrans, bestMatch = rdMolAlign.GetBestAlignmentTransform(
      prbCopy2, ref, map=matchesPruned)
    self.assertEqual(str(type(bestTrans)), "<class 'numpy.ndarray'>")
    self.assertAlmostEqual(rmsd, rmsdCopy, 3)
    self.assertEqual(len(bestMatch), len(scaffoldMatch))
    weights = [100.0 if bit else 1.0 for bit in scaffoldIndicesBitSet]
    rmsdInPlace = rdMolAlign.CalcRMS(prbCopy3, ref, map=matches, weights=weights)
    self.assertAlmostEqual(rmsdInPlace, 17.7959, 3)
    rmsd = rdMolAlign.GetBestRMS(prb, ref, map=matches, weights=weights)
    self.assertAlmostEqual(rmsd, 10.9681, 3)
    rmsdCopy, bestTrans, bestMatch = rdMolAlign.GetBestAlignmentTransform(
      prbCopy3, ref, map=matches, weights=weights)
    self.assertAlmostEqual(rmsd, rmsdCopy, 3)
    self.assertEqual(len(bestMatch), ref.GetNumAtoms())
    self.assertTrue(all(len(tup) == 2 for tup in bestMatch))

  def test18GetBestRMSAndConjugatedGroups(self):
    mol = Chem.MolFromSmiles(
      "CCC(=O)[O-] |(-1.11,0.08,-0.29;0.08,-0.18,0.58;1.34,0.03,-0.16;1.74,1.22,-0.32;2.06,-1.04,-0.66)|"
    )
    qry = Chem.MolFromSmiles(
      "CCC([O-])=O |(-1.11,0.08,-0.29;0.08,-0.18,0.58;1.34,0.03,-0.16;1.74,1.22,-0.32;2.06,-1.04,-0.66)|"
    )

    rmsd = rdMolAlign.GetBestRMS(qry, mol)
    self.assertAlmostEqual(rmsd, 0, 3)

    rmsd = rdMolAlign.GetBestRMS(qry, mol, symmetrizeConjugatedTerminalGroups=False)
    self.assertAlmostEqual(rmsd, 0.747, 3)

  def test19GetAllConformerBestRMS(self):
    file1 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolAlign', 'test_data',
                         'symmetric.confs.sdf')
    ms = [x for x in Chem.SDMolSupplier(file1)]
    mol = Chem.Mol(ms[0])
    for i in range(1, len(ms)):
      mol.AddConformer(ms[i].GetConformer(), assignId=True)

    nconfs = mol.GetNumConformers()
    origVals = rdMolAlign.GetAllConformerBestRMS(mol)
    self.assertEqual(len(origVals), (nconfs * (nconfs - 1)) // 2)

    self.assertAlmostEqual(origVals[0], rdMolAlign.GetBestRMS(mol, mol, 0, 1))

    newVals = rdMolAlign.GetAllConformerBestRMS(mol, numThreads=4)
    self.assertEqual(len(origVals), len(newVals))
    for ov, nv in zip(origVals, newVals):
      self.assertAlmostEqual(ov, nv)


if __name__ == '__main__':
  print("Testing MolAlign Wrappers")
  unittest.main()



#
#  Copyright (C) 2019 Greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import argparse
import gzip
import time

import rdkit
from rdkit import Chem, DataStructs
from rdkit.Chem import AllChem
from rdkit.RDLogger import logger

logger = logger()

parser = argparse.ArgumentParser(
  description="benchmark and test fingerprint screenout and substructure searching")
parser.add_argument("--validate", dest='validateResults', default=False, action='store_true',
                    help="validate that the screenout isn't missing anything")
parser.add_argument("--short", dest='doShort', default=False, action='store_true',
                    help="run a small subset of the molecules")
args = parser.parse_args()

ts = []

logger.info('mols from smiles')
mols = []
t1 = time.time()
# find this file here: https://raw.githubusercontent.com/greglandrum/rdkit_blog/master/data/chembl21_25K.pairs.txt.gz
with gzip.open('../Data/chembl21_25K.pairs.txt.gz', 'rb') as inf:
  for line in inf:
    line = line.decode().strip().split()
    smi1 = line[1]
    smi2 = line[3]
    mols.append(Chem.MolFromSmiles(smi1))
    mols.append(Chem.MolFromSmiles(smi2))
    if args.doShort and len(mols) >= 1000:
      break
t2 = time.time()
ts.append(t2 - t1)
logger.info(f'Results{len(ts)}: {t2-t1 : .2f} seconds, {len(mols)} mols')

logger.info('queries from smiles')
t1 = time.time()
# find this file here: https://raw.githubusercontent.com/greglandrum/rdkit_blog/master/data/zinc.frags.500.q.smi
frags = [Chem.MolFromSmiles(x.split()[0]) for x in open('../Data/zinc.frags.500.q.smi', 'r')]
# find this file here: https://raw.githubusercontent.com/greglandrum/rdkit_blog/master/data/zinc.leads.500.q.smi
leads = [Chem.MolFromSmiles(x.split()[0]) for x in open('../Data/zinc.leads.500.q.smi', 'r')]
# find this file here: https://raw.githubusercontent.com/greglandrum/rdkit_blog/master/data/fragqueries.q.txt
pieces = [Chem.MolFromSmiles(x) for x in open('../Data/fragqueries.q.txt', 'r')]
t2 = time.time()
ts.append(t2 - t1)
logger.info(f'Results{len(ts)}: {t2-t1 : .2f} seconds')

logger.info('generating pattern fingerprints for mols')
t1 = time.time()
mfps = [Chem.PatternFingerprint(m) for m in mols]
t2 = time.time()
ts.append(t2 - t1)
logger.info(f'Results{len(ts)}: {t2-t1 : .2f} seconds')

logger.info('generating pattern fingerprints for queries')
t1 = time.time()
fragsfps = [Chem.PatternFingerprint(m, 2048) for m in frags]
leadsfps = [Chem.PatternFingerprint(m, 2048) for m in leads]
piecesfps = [Chem.PatternFingerprint(m, 2048) for m in pieces]
t2 = time.time()
ts.append(t2 - t1)
logger.info(f'Results{len(ts)}: {t2-t1 : .2f} seconds')

for nm, qs, qfps in [('frags', frags, fragsfps), ('leads', leads, leadsfps),
                     ('pieces', pieces, piecesfps)]:
  logger.info(f'testing {nm} queries')
  t1 = time.time()
  nPossible = 0
  nTested = 0
  nFound = 0
  nErrors = 0
  for i, fragfp in enumerate(qfps):
    for j, mfp in enumerate(mfps):
      nPossible += 1
      if args.validateResults:
        matched = mols[j].HasSubstructMatch(qs[i])
        fpMatch = DataStructs.AllProbeBitsMatch(fragfp, mfp)
        if fpMatch:
          nTested += 1
        if matched:
          nFound += 1
          if not fpMatch:
            nErrors += 1
            logger.error(f"ERROR: mol {j} query {i}")
      else:
        if DataStructs.AllProbeBitsMatch(fragfp, mfp):
          nTested += 1
          if mols[j].HasSubstructMatch(qs[i]):
            nFound += 1
  t2 = time.time()
  ts.append(t2 - t1)
  logger.info(
    f'Results{len(ts)}: {t2-t1 : .2f} seconds. {nTested} tested ({nTested/nPossible :.4f} of total), {nFound} found, {nFound/nTested : .2f} accuracy. {nErrors} errors.'
  )

print(f"| {rdkit.__version__} | {' | '.join(['%.1f' % x for x in ts])} |")



#  Copyright (c) 2023 David Cosgrove and other RDKit contributors
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# These tests are just to check that the Python wrappers are working
# ok.  The bulk of the tests are in the C++ code.
import os
import unittest

from pathlib import Path

from rdkit import Chem
from rdkit.Chem import rdRascalMCES


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1(self):
    mol1 = Chem.MolFromSmiles("c1ccccc1Cl")
    mol2 = Chem.MolFromSmiles("c1ccccc1F")
    opts = rdRascalMCES.RascalOptions()

    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0].smartsString, 'c1:c:c:c:c:c:1')
    self.assertEqual(len(results[0].bondMatches()), 6)
    self.assertEqual(len(results[0].atomMatches()), 6)

  def test2(self):
    # Test single largest fragment extraction from results
    ad1 = Chem.MolFromSmiles("CN(C)c1ccc(CC(=O)NCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL153934")
    ad2 = Chem.MolFromSmiles("N(C)c1ccc(CC(=O)NCCCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL157336")

    opts = rdRascalMCES.RascalOptions()
    results = rdRascalMCES.FindMCES(ad1, ad2, opts)
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0].smartsString,
                     'N(-C)-c1:c:c:c(-CC(=O)-NCCCCCCCCCC):c:c:1.NC12CC3CC(-C1)-CC(-C2)-C3')
    results[0].largestFragmentOnly()
    self.assertEqual(results[0].smartsString, 'N(-C)-c1:c:c:c(-CC(=O)-NCCCCCCCCCC):c:c:1')

  def test3(self):
    # Test not specifying options
    mol1 = Chem.MolFromSmiles("c1ccccc1Cl")
    mol2 = Chem.MolFromSmiles("c1ccccc1F")

    results = rdRascalMCES.FindMCES(mol1, mol2)
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0].smartsString, 'c1:c:c:c:c:c:1')
    self.assertEqual(len(results[0].bondMatches()), 6)
    self.assertEqual(len(results[0].atomMatches()), 6)

  def test4(self):
    # Test setting non-default option
    mol1 = Chem.MolFromSmiles('Oc1cccc2C(=O)C=CC(=O)c12')
    mol2 = Chem.MolFromSmiles('O1C(=O)C=Cc2cc(OC)c(O)cc12')
    results = rdRascalMCES.FindMCES(mol1, mol2)
    self.assertEqual(len(results), 0)

    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.5
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(len(results), 1)

  def test5(self):
    # Test setting non-default option singleLargestFrag
    ad1 = Chem.MolFromSmiles("CN(C)c1ccc(CC(=O)NCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL153934")
    ad2 = Chem.MolFromSmiles("N(C)c1ccc(CC(=O)NCCCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL157336")

    opts = rdRascalMCES.RascalOptions()
    opts.singleLargestFrag = True
    results = rdRascalMCES.FindMCES(ad1, ad2, opts)
    self.assertEqual(len(results), 1)
    self.assertEqual(results[0].smartsString, 'CCCCCCCCCCNC12CC3CC(-C1)-CC(-C2)-C3')

  def test6(self):
    # Test the threshold and examine the tier1 and tier2 similarities.
    ad1 = Chem.MolFromSmiles("CN(C)c1ccc(CC(=O)NCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL153934")
    ad2 = Chem.MolFromSmiles("N(C)c1ccc(CC(=O)NCCCCCCCCCCCCNC23CC4CC(C2)CC(C3)C4)cc1 CHEMBL157336")

    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.95
    results = rdRascalMCES.FindMCES(ad1, ad2, opts)
    self.assertEqual(len(results), 0)
    opts.returnEmptyMCES = True
    results = rdRascalMCES.FindMCES(ad1, ad2, opts)
    self.assertEqual(len(results), 1)

  def testRascalCluster(self):
    cdk2_file = Path(os.environ['RDBASE']) / 'Contrib' / 'Fastcluster' / 'cdk2.smi'
    suppl = Chem.SmilesMolSupplier(str(cdk2_file), '\t', 1, 0, False)
    mols = [mol for mol in suppl]
    clusters = rdRascalMCES.RascalCluster(mols)
    self.assertEqual(len(clusters), 8)
    expClusters = [7, 7, 6, 2, 2, 2, 2, 20]
    for clus, expClusSize in zip(clusters, expClusters):
      self.assertEqual(expClusSize, len(clus))

    clusOpts = rdRascalMCES.RascalClusterOptions()
    clusOpts.similarityCutoff = 0.6
    clusters = rdRascalMCES.RascalCluster(mols, clusOpts)
    expClusters = [9, 8, 6, 2, 2, 2, 2, 2, 2, 2, 11]
    for clus, expClusSize in zip(clusters, expClusters):
      self.assertEqual(expClusSize, len(clus))

  def testRascalButinaCluster(self):
    cdk2_file = Path(os.environ['RDBASE']) / 'Contrib' / 'Fastcluster' / 'cdk2.smi'
    suppl = Chem.SmilesMolSupplier(str(cdk2_file), '\t', 1, 0, False)
    mols = [mol for mol in suppl]
    clusters = rdRascalMCES.RascalButinaCluster(mols)
    self.assertEqual(len(clusters), 29)
    expClusters = [
      6, 6, 6, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
    ]
    for clus, expClusSize in zip(clusters, expClusters):
      self.assertEqual(expClusSize, len(clus))

  def testMaxBondMatchPairs(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.0
    opts.returnEmptyMCES = True
    opts.singleLargestFrag = True
    opts.allBestMCESs = True
    opts.completeAromaticRings = False
    opts.timeout = -1

    too_long_1 =  Chem.MolFromSmiles('CCCC=CCCCC=CCCCCCCCCCCCCCCCCCCCCC1CNCCC1')
    too_long_2 =  Chem.MolFromSmiles('CCCC=CCCCCC=CCCCCCCCCCCCCCCCCCCCC1CNCCC1')
    results = rdRascalMCES.FindMCES(too_long_1, too_long_2, opts)
    self.assertEqual(len(results[0].bondMatches()), 0)

    opts.maxBondMatchPairs = 1200
    results = rdRascalMCES.FindMCES(too_long_1, too_long_2, opts)
    self.assertEqual(len(results[0].bondMatches()), 26)

  def testExactConnectionsMatch(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.5
    opts.allBestMCESs = True
    mol1 = Chem.MolFromSmiles('c1ccccc1C1CCC(C(C)C)C1')
    mol2 = Chem.MolFromSmiles('c1ccccc1C(C)C')
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(results[0].numFragments, 1)
    self.assertEqual(results[0].smartsString, 'c1:c:c:c:c:c:1-C(-C)-C')

    opts.exactConnectionsMatch = True
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(results[0].numFragments, 2)
    self.assertEqual(results[0].smartsString,
                     '[#6&a&D2]1:[#6&a&D2]:[#6&a&D2]:[#6&a&D2]:[#6&a&D2]:[#6&a&D3]:1.[#6&A&D3](-[#6&A&D1])-[#6&A&D1]')

  def testEquivalentAtoms(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.5
    opts.equivalentAtoms = "[F,Cl,Br,I]"
    mol1 = Chem.MolFromSmiles('c1ccccc1F')
    mol2 = Chem.MolFromSmiles('c1ccccc1Br')
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(results[0].numFragments, 1)
    self.assertEqual(results[0].smartsString, 'c1:c:c:c:c:c:1-[F,Cl,Br,I]')

  def testEquivalentBonds(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.5
    opts.ignoreBondOrders = True
    mol1 = Chem.MolFromSmiles('CC=CC')
    mol2 = Chem.MolFromSmiles('CCCC')
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(results[0].numFragments, 1)
    self.assertEqual(results[0].smartsString, 'C~C~C~C')
    

  def testExactAtomTypeMatch(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.1
    opts.ignoreAtomAromaticity = False
    mol1 = Chem.MolFromSmiles('c1ccccc1NCC')
    mol2 = Chem.MolFromSmiles('C1CCCCC1NCC')
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertEqual(results[0].numFragments, 1)
    self.assertEqual(results[0].smartsString, 'NCC')

  def testMinCliqueSize(self):
    opts = rdRascalMCES.RascalOptions()
    opts.similarityThreshold = 0.1
    opts.minCliqueSize = 17
    mol1 = Chem.MolFromSmiles('CC12CCC3C(C1CCC2O)CCC4=CC(=O)CCC34C')
    mol2 = Chem.MolFromSmiles('CC12CCC3C(C1CCC2O)CCC4=C3C=CC(=C4)O')
    results = rdRascalMCES.FindMCES(mol1, mol2, opts)
    self.assertFalse(results)

    
if __name__ == "__main__":
  unittest.main()



import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import rdMMPA


def natoms(tpl):
  return tuple(x.GetNumAtoms() if x is not None else 0 for x in tpl)


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1(self):
    m = Chem.MolFromSmiles('c1ccccc1OC')
    frags = rdMMPA.FragmentMol(m)
    self.assertEqual(len(frags), 3)
    for frag in frags:
      self.assertEqual(len(frag), 2)
    frags = sorted(frags, key=natoms)
    self.assertEqual(frags[0][0], None)
    self.assertEqual(frags[1][0], None)
    self.assertNotEqual(frags[2][0], None)
    self.assertNotEqual(frags[0][1], None)
    self.assertNotEqual(frags[1][1], None)
    self.assertNotEqual(frags[2][1], None)

    self.assertEqual(frags[0][1].GetNumAtoms(), m.GetNumAtoms() + 2)
    self.assertEqual(frags[1][1].GetNumAtoms(), m.GetNumAtoms() + 2)

    fs = Chem.GetMolFrags(frags[0][1], asMols=True)
    self.assertEqual(len(fs), 2)
    self.assertEqual(Chem.MolToSmiles(fs[0], True), 'c1ccc([*:1])cc1')
    self.assertEqual(Chem.MolToSmiles(fs[1], True), 'CO[*:1]')

    fs = Chem.GetMolFrags(frags[1][1], asMols=True)
    self.assertEqual(len(fs), 2)
    self.assertEqual(Chem.MolToSmiles(fs[0], True), 'c1ccc(O[*:1])cc1')
    self.assertEqual(Chem.MolToSmiles(fs[1], True), 'C[*:1]')

    fs = Chem.GetMolFrags(frags[2][0], asMols=True)
    self.assertEqual(len(fs), 1)
    self.assertEqual(Chem.MolToSmiles(fs[0], True), 'O([*:1])[*:2]')
    fs = Chem.GetMolFrags(frags[2][1], asMols=True)
    self.assertEqual(len(fs), 2)
    self.assertEqual(Chem.MolToSmiles(fs[0], True), 'c1ccc([*:2])cc1')
    self.assertEqual(Chem.MolToSmiles(fs[1], True), 'C[*:1]')

  def test2(self):
    m = Chem.MolFromSmiles('c1ccccc1OC')
    frags = rdMMPA.FragmentMol(m, resultsAsMols=False)
    self.assertEqual(len(frags), 3)
    for frag in frags:
      self.assertEqual(len(frag), 2)
    frags = sorted(frags)
    self.assertEqual(frags[0][0], '')
    self.assertEqual(frags[1][0], '')
    self.assertNotEqual(frags[2][0], '')
    self.assertNotEqual(frags[0][1], '')
    self.assertNotEqual(frags[1][1], '')
    self.assertNotEqual(frags[2][1], '')

    self.assertEqual(frags[0][1], 'CO[*:1].c1ccc([*:1])cc1')
    self.assertEqual(frags[1][1], 'C[*:1].c1ccc(O[*:1])cc1')
    self.assertEqual(frags[2][0], 'O([*:1])[*:2]')
    self.assertEqual(frags[2][1], 'C[*:1].c1ccc([*:2])cc1')

  def test3(self):
    m = Chem.MolFromSmiles('c1ccccc1OC')
    frags = rdMMPA.FragmentMol(m, resultsAsMols=False, pattern='cO')
    self.assertEqual(len(frags), 1)
    for frag in frags:
      self.assertEqual(len(frag), 2)
    frags = sorted(frags)
    self.assertEqual(frags[0][0], '')
    self.assertNotEqual(frags[0][1], '')

    self.assertEqual(frags[0][1], 'CO[*:1].c1ccc([*:1])cc1')

  def test4(self):
    m = Chem.MolFromSmiles('Cc1ccccc1NC(=O)C(C)[NH+]1CCCC1')  # ZINC00000051
    frags = rdMMPA.FragmentMol(m, resultsAsMols=False)
    #for frag in sorted(frags):
    #    print(frag)
    cores = set(x[0] for x in frags)
    self.assertTrue('C([*:1])([*:2])[*:3]' in cores)
    # FIX: this needs to be investigated, it's not currently passing
    #self.assertTrue('O=C(N[*:3])C([*:1])[*:2]' in cores)
    self.assertEqual(len(frags), 18)
    for frag in frags:
      self.assertEqual(len(frag), 2)

  def test5(self):
    m = Chem.MolFromSmiles(
      "CC[C@H](C)[C@@H](C(=O)N[C@H]1CSSC[C@H]2C(=O)NCC(=O)N3CCC[C@H]3C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@@H](CSSC[C@@H](C(=O)N[C@H](C(=O)N4CCC[C@H]4C(=O)N[C@H](C(=O)N2)C)CC(=O)N)NC1=O)C(=O)N)CO)Cc5ccc(cc5)O)CCCC[NH3+])N"
    )  # ALPHA-CONOTOXIN SI
    frags = rdMMPA.FragmentMol(m, resultsAsMols=False)
    self.assertFalse(len(frags))
    frags = rdMMPA.FragmentMol(m, maxCuts=2, maxCutBonds=21, resultsAsMols=False)
    self.assertEqual(len(frags), 231)

  def test6(self):
    m = Chem.MolFromSmiles(
      "CC[C@H](C)[C@@H](C(=O)N[C@H]1CSSC[C@H]2C(=O)NCC(=O)N3CCC[C@H]3C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@@H](CSSC[C@@H](C(=O)N[C@H](C(=O)N4CCC[C@H]4C(=O)N[C@H](C(=O)N2)C)CC(=O)N)NC1=O)C(=O)N)CO)Cc5ccc(cc5)O)CCCC[NH3+])N"
    )  # ALPHA-CONOTOXIN SI
    frags = rdMMPA.FragmentMol(m, resultsAsMols=False)
    self.assertFalse(len(frags))
    frags1 = rdMMPA.FragmentMol(m, minCuts=1, maxCuts=1, maxCutBonds=21, resultsAsMols=False)
    frags2 = rdMMPA.FragmentMol(m, minCuts=2, maxCuts=2, maxCutBonds=21, resultsAsMols=False)

    frags = rdMMPA.FragmentMol(m, maxCuts=2, maxCutBonds=21, resultsAsMols=False)

    self.assertEqual(set(frags1 + frags2), set(frags))
    self.assertEqual(set(frags1).intersection(set(frags2)), set())

  def test7(self):
    m = Chem.MolFromSmiles("Oc1ccccc1N")

    frags1 = rdMMPA.FragmentMol(m, minCuts=1, maxCuts=1, maxCutBonds=21, resultsAsMols=False)

    frags2 = rdMMPA.FragmentMol(m, minCuts=2, maxCuts=2, maxCutBonds=21, resultsAsMols=False)

    frags = rdMMPA.FragmentMol(m, maxCuts=2, maxCutBonds=21, resultsAsMols=False)

    self.assertEqual(set(frags1 + frags2), set(frags))

  def test8(self):
    m = Chem.MolFromSmiles('Cc1ccccc1NC(=O)C(C)[NH+]1CCCC1')  # ZINC00000051
    sm = Chem.MolFromSmarts("[#6+0;!$(*=,#[!#6])]!@!=!#[*]")
    matching_atoms = m.GetSubstructMatches(sm)
    bonds = []
    for a, b in matching_atoms:
      bond = m.GetBondBetweenAtoms(a, b)
      bonds.append(bond.GetIdx())

    frags = rdMMPA.FragmentMol(m, resultsAsMols=False)
    frags2 = rdMMPA.FragmentMol(m, bonds, resultsAsMols=False)
    frags3 = rdMMPA.FragmentMol(m, tuple(bonds), resultsAsMols=False)
    self.assertEqual(frags, frags2)
    self.assertEqual(frags2, frags3)

  def test9(self):
    m = Chem.MolFromSmiles("Oc1ccccc1N")

    try:

      frags1 = rdMMPA.FragmentMol(m, minCuts=1, maxCuts=0, maxCutBonds=21, resultsAsMols=False)
      self.assertTrue(False)  # should not get here
    except ValueError as e:
      self.assertEqual(str(e), "supplied maxCuts is less than minCuts")

    try:

      frags1 = rdMMPA.FragmentMol(m, minCuts=0, maxCuts=0, maxCutBonds=21, resultsAsMols=False)
      self.assertTrue(False)  # should not get here
    except ValueError as e:
      self.assertEqual(str(e), "minCuts must be greater than 0")


if __name__ == "__main__":
  unittest.main()



# $Id$
#
# Copyright (C) 2003-2006 greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" unit testing code for BitEnsembles
"""
import os
import shutil
import tempfile
import unittest

from rdkit import RDConfig
# This import is important to initialize the BitEnsemble module
from rdkit.DataStructs import BitEnsembleDb, SparseBitVect
from rdkit.DataStructs.BitEnsemble import BitEnsemble


class TestCase(unittest.TestCase):

  def test1(self):
    ensemble = BitEnsemble()
    ensemble.SetBits([1, 11, 21, 31])
    self.assertEqual(ensemble.GetNumBits(), 4)
    bv = SparseBitVect(100)
    bv.SetBit(1)
    bv.SetBit(11)
    bv.SetBit(13)

    score = ensemble.ScoreWithOnBits(bv)
    assert score == 2, 'bad score: %d' % (score)
    score = ensemble.ScoreWithIndex(bv)
    assert score == 2, 'bad score: %d' % (score)

  def test2(self):
    ensemble = BitEnsemble([1, 11, 21, 31])
    bv = SparseBitVect(100)
    bv.SetBit(1)
    bv.SetBit(11)
    bv.SetBit(13)

    score = ensemble.ScoreWithOnBits(bv)
    assert score == 2, 'bad score: %d' % (score)
    score = ensemble.ScoreWithIndex(bv)
    assert score == 2, 'bad score: %d' % (score)

  def test3(self):
    ensemble = BitEnsemble()
    for bit in [1, 11, 21, 31]:
      ensemble.AddBit(bit)
    bv = SparseBitVect(100)
    bv.SetBit(1)
    bv.SetBit(11)
    bv.SetBit(13)

    score = ensemble.ScoreWithOnBits(bv)
    assert score == 2, 'bad score: %d' % (score)
    score = ensemble.ScoreWithIndex(bv)
    assert score == 2, 'bad score: %d' % (score)

  def _setupDb(self):
    from rdkit.Dbase.DbConnection import DbConnect
    fName = RDConfig.RDTestDatabase
    if RDConfig.useSqlLite:
      _, tempName = tempfile.mkstemp(suffix='sqlt')
      self.tempDbName = tempName
      shutil.copyfile(fName, tempName)
    else:  # pragma: nocover
      tempName = '::RDTests'
    self.conn = DbConnect(tempName)
    self.dbTblName = 'bit_ensemble_test'
    return self.conn

  def tearDown(self):
    if hasattr(self, 'tempDbName') and RDConfig.useSqlLite and os.path.exists(self.tempDbName):
      try:
        os.unlink(self.tempDbName)
      except Exception:  # pragma: nocover
        import traceback
        traceback.print_exc()

  def testdb1(self):
    """ test the sig - db functionality """
    conn = self._setupDb()
    ensemble = BitEnsemble()
    for bit in [1, 3, 4]:
      ensemble.AddBit(bit)

    sigBs = [
      ([0, 0, 0, 0, 0, 0], (0, 0, 0)),
      ([0, 1, 0, 1, 0, 0], (1, 1, 0)),
      ([0, 1, 0, 0, 1, 0], (1, 0, 1)),
      ([0, 1, 0, 0, 1, 1], (1, 0, 1)),
    ]
    ensemble.InitScoreTable(conn, self.dbTblName)
    for bs, tgt in sigBs:
      ensemble.ScoreToDb(bs, conn)
    conn.Commit()

    d = conn.GetData(table=self.dbTblName)
    assert len(d) == len(sigBs), 'bad number of results returned'
    for i in range(len(sigBs)):
      bs, tgt = tuple(sigBs[i])
      dbRes = tuple(d[i])
      assert dbRes == tgt, 'bad bits returned: %s != %s' % (str(dbRes), str(tgt))
    d = None
    self.conn = None

  def testdb2(self):
    """ test the sig - db functionality """
    conn = self._setupDb()
    ensemble = BitEnsemble()
    for bit in [1, 3, 4]:
      ensemble.AddBit(bit)

    sigBs = [
      ([0, 0, 0, 0, 0, 0], (0, 0, 0)),
      ([0, 1, 0, 1, 0, 0], (1, 1, 0)),
      ([0, 1, 0, 0, 1, 0], (1, 0, 1)),
      ([0, 1, 0, 0, 1, 1], (1, 0, 1)),
    ]
    ensemble.InitScoreTable(conn, self.dbTblName, idInfo='id varchar(10)', actInfo='act int')
    for bs, tgt in sigBs:
      ensemble.ScoreToDb(bs, conn, id='foo', act=1)
    conn.Commit()

    d = conn.GetData(table=self.dbTblName)
    assert len(d) == len(sigBs), 'bad number of results returned'
    for i in range(len(sigBs)):
      bs, tgt = tuple(sigBs[i])
      dbRes = tuple(d[i])
      assert dbRes[1:-1] == tgt, 'bad bits returned: %s != %s' % (str(dbRes[1:-1]), str(tgt))

    d = None
    self.conn = None


if __name__ == '__main__':  # pragma: nocover
  unittest.main()
